// FILE: ./assets/js/InflightorGeneral.js
var InflightorGeneral = function(ScreenService, templateManager, localStorageEngine, restEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory, ZhFilterPreferences, engine, eventAggregator, General, MessageService) {
    var searchListDeleted;
    var parent = this;

    var INDEX_NOT_FOUND = -1;

    General.initRoutesListener();
    //@TODO Criar objeto de mensagens para tradução.
    /*
     * Função que atualiza o dataSourceFilter de um field que depende do resultado de
     * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
     * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
     * ao alterar o valor do field "pai".
     *
     * @param field dependentField
     * @param array parameters
     *
     * Exemplo:
     * InflightorGeneral.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
     */

    eventAggregator.subscribe("onContainerOpen", _validateInflightorSession);

    function _validateInflightorSession() {
        return engine.getStartSessionDefer().then(function() {
            let cdoperador = getLocalVar('INFLIGHTOR_CDOPERADOR') || getLocalVar('CDOPERADOR');
            setLocalVar('INFLIGHTOR_CDOPERADOR', cdoperador);

            let nrorgtrab = getLocalVar('INFLIGHTOR_NRORGTRAB') || getLocalVar('NRORGTRAB');
            setLocalVar('INFLIGHTOR_NRORGTRAB', nrorgtrab);

            let cdfilial = getLocalVar('INFLIGHTOR_CDFILIAL') || getLocalVar('CDFILIAL');
            setLocalVar('INFLIGHTOR_CDFILIAL', cdfilial);

            if (!cdoperador || !nrorgtrab) {
                engine.disconnect();
            } else if (!cdfilial) {
                engine.openWindow('cadEst#ger01742_selecionar_unidade');
            }
        });
    }
    this.validateInflightorSession = _validateInflightorSession;

    this.openFilterOnEnter = function(widgetToFilter, fieldsToApplyDefaultValue = []) {
        var filterRow = {};
        fieldsToApplyDefaultValue.forEach(function(fieldName) {
            let field = widgetToFilter.getField(fieldName);
            if (field) {
                this.applyFieldDefaultValueOnFilterPopup(widgetToFilter, field);
                filterRow[fieldName] = field.value();
            }
        }.bind(this));
        ZhFilterPreferences.setOriginalFilter(widgetToFilter.id, filterRow);
        ZhFilterPreferences.openFilterWidget(widgetToFilter, filterRow);
    };

    this.applyFieldDefaultValueOnFilterPopup = function(filterWidget, field) {

        var fieldDefaultValue;

        if (!Util.isEmptyOrBlank(field.filterProperties) && field.filterProperties.hasOwnProperty("defaultValue")) {
            fieldDefaultValue = field.filterProperties.defaultValue;
        } else {
            fieldDefaultValue = field.defaultValue;
        }

        if (filterWidget.currentRow) {
            filterWidget.currentRow[field.name] = typeof fieldDefaultValue === "undefined" ? "" : ZhUtil.evaluate(fieldDefaultValue);
        }
    };

    var getStripeLabel = function(widget) {
        if (widget && widget._filterInfo) {
            var formatedString = widget._filterInfo.reduce((acc, cur) => acc + cur.label + ": " + cur.value + " • ", "");
            if (widget._filterInfo.length > 0) {
                formatedString = formatedString.substring(0, formatedString.length - 3);
            }
            return formatedString;
        } else {
            return "";
        }
    };

    var updateStripe = function(gridWidget) {
        var deferred = ZHPromise.defer();

        var stripeFilterFields = [];
        var filterInfo = gridWidget.__filterInfoStripe;

        if (!filterInfo) {
            handleStripe(gridWidget).then(function() {
                filterInfo = gridWidget.__filterInfoStripe;
                deferred.resolve();
            });
        } else {
            deleteUsedFilterParameterOnStripeFields(filterInfo);
            setFieldsByFilterOnStripe(gridWidget._filterInfo, filterInfo);
            hideFieldsNotUsedOnStripe(filterInfo);
            activateCollapseButtonByVisibleFieldsOnStripe(filterInfo);
            deferred.resolve();
        }

        return deferred.promise;
    };

    this.updateStripe = updateStripe;

    /**
     * Deleta o parâmetro que indica quando uma field do stripe
     * já foi usada para exibir indormações do filtro definido
     * (em todos os fields do stripe).
     * 
     * @param {object} stripe widgtet stripe.
     */
    function deleteUsedFilterParameterOnStripeFields(stripe) {

        stripe.fields.forEach(function(field) {

            if (field.hasOwnProperty("usedFiedlOnStripe")) {
                delete field.usedFiedlOnStripe;
            }

        });

    }

    /**
     * Define a rotina para exibir os valores filtrados
     * no stripe, onde "OBJECTS_BY_LABEL_ON_STRIPE" e a variável
     * que define o máximo de informações do filtro atual a serem
     * setadas por field.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function setFieldsByFilterOnStripe(filterInfo, stripe) {

        if (!Util.isEmptyOrBlank(filterInfo)) {

            var validfilterInfo = getValidFilterInfo(filterInfo);

            var OBJECTS_BY_LABEL_ON_STRIPE = 3;
            var filterInfoLength = validfilterInfo.length;

            if (filterInfoLength > OBJECTS_BY_LABEL_ON_STRIPE) {

                buildStripeField(validfilterInfo.slice(0, 3), stripe);
                setFieldsByFilterOnStripe(validfilterInfo.slice(3), stripe);

            } else if (filterInfoLength > 0) {
                buildStripeField(validfilterInfo, stripe);
            }

        }

    }

    /**
     * Retonar um array com os valores válidos
     * a serem definidos no stripe, ou seja, retorna
     * apenas as informações que possuem label.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     */
    function getValidFilterInfo(filterInfo) {

        let labelsToExclude = ["TODAS", "TODOS"];

        var validFilterInfo = filterInfo.filter(function(filterInfo) {
            let valueIsFilled = (!Util.isEmptyOrBlank(filterInfo.value));
            let labelIsFilled = (!Util.isEmptyOrBlank(filterInfo.label));

            let upperCaseLabel = filterInfo.label.toUpperCase();
            let labelIsNotInExcludeList = (labelsToExclude.indexOf(upperCaseLabel) == -1);

            return (valueIsFilled && labelIsFilled && labelIsNotInExcludeList);
        });
        return validFilterInfo;
    }

    /**
     * Cria um field, ou utiliza um já existente,
     * para ser definido no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildStripeField(filterInfo, stripe) {

        var stripeFields = stripe.fields;
        var availableFieldIndex = getAvailableFieldIndex(stripeFields);

        if (availableFieldIndex !== undefined) {

            var stripeField = stripeFields[availableFieldIndex];
            var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo, stripe);
            stripeField.label = fieldLabel;

            if (stripeField.isVisible === false) {
                stripeField.isVisible = true;
            }

        } else {
            stripeFields.push(buildFieldForStripe(filterInfo, stripe));
        }

    }

    /**
     * Retonar o index de um field que ainda não foi utilizado,
     * no stripe, para exibir valores do filtro.
     * 
     * @param {object} stripeFields fields existentes no stripe.
     */
    function getAvailableFieldIndex(stripeFields) {

        if (stripeFields !== undefined && stripeFields.length > 0) {

            for (var i = 0; i < stripeFields.length; i++) {

                if (!Boolean(stripeFields[i].usedFiedlOnStripe)) {

                    stripeFields[i].usedFiedlOnStripe = true;
                    return i;

                }

            }

        }

        return undefined;

    }

    /**
     * Cria um field para exibir informações do filtro
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * definido na widget.
     * 
     * @param {object} stripe widgtet stripe.
     */
    function buildFieldForStripe(filterInfo, stripe) {

        var fieldLabel = buildStripeFielLabelByFilterInfo(filterInfo);

        var field = {
            name: "STRIPE_FIELD",
            label: fieldLabel,
            widget: stripe,
            usedFiedlOnStripe: true
        };

        return metaDataFactory.fieldFactory(field);

    }

    /**
     * Cria o texto que será definido na label de um field
     * no stripe.
     * 
     * @param {object} filterInfo informações do filtro
     * que serão utilizadas para montar o texto.
     */
    function buildStripeFielLabelByFilterInfo(filterInfo) {

        var fieldLabel;

        if (filterInfo.length === 1) {
            fieldLabel = filterInfo[0].label + ": " + filterInfo[0].value;
        } else {

            fieldLabel = filterInfo.reduce(function(acc, cur) {
                return acc + cur.label + ": " + cur.value + " • ";
            }, "");

        }

        return fieldLabel;

    }

    /**
     * Oculta fields que não são usandos no stripe.
     * 
     * @param {object} stripe widget stripe.
     */
    function hideFieldsNotUsedOnStripe(stripe) {

        stripe.fields.forEach(function(field) {

            if (!field.hasOwnProperty("usedFiedlOnStripe")) {
                field.isVisible = false;
            }

        });

    }

    function activateCollapseButtonByVisibleFieldsOnStripe(stripe) {

        var countStripeVisibleFields = 0;
        var NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON = 3;
        var stripeFields = stripe.fields;

        stripeFields.forEach(function(field) {

            if (field.isVisible) {
                countStripeVisibleFields++;
            }

        });

        if (countStripeVisibleFields >= NUMBER_OF_FIELDS_TO_SHOW_COLLAPSE_BUTTON) {
            stripe.collapsible = true;
        } else {
            stripe.collapsible = false;
        }

    }

    var handleStripe = function(gridWidget) {
        var deferred = ZHPromise.defer();
        var stripeName = gridWidget.stripeToShowGridFilterInfo;
        if (stripeName) {
            var stripeWidget = templateManager.container.getWidget(stripeName);
            if (stripeWidget) {
                if (~stripeWidget.template.indexOf("stripe.html")) {
                    var stripe = gridWidget.__filterInfoStripe = stripeWidget;
                    stripe.__stripeOriginalFields = stripe.fields;
                    factoryStripeFields([], gridWidget).then(function(fields) {
                        stripe.fields = fields;
                        deferred.resolve();
                    });
                } else {
                    console.error("ZhGridFilter: Widget " + stripeName + " is not a Stripe");
                    deferred.reject();
                }
            } else {
                console.error("ZhGridFilter: There is no widget named " + stripeName);
                deferred.reject();
            }
        }
        return deferred.promise;
    };

    var factoryStripeFields = function(fields, gridWidget) {
        var deferred = ZHPromise.defer();
        var stripe = gridWidget.__filterInfoStripe;
        fields = stripe.__stripeOriginalFields ? fields.concat(stripe.__stripeOriginalFields) : fields;
        var fieldsMap = fields.map(function(field) {
            var factoredField = metaDataFactory.fieldFactory(field);
            factoredField.widget = stripe;
            return factoredField;
        });
        deferred.resolve(fields);
        return deferred.promise;
    };

    ZhFilterPreferences.oldApplyFilter = ZhFilterPreferences.applyFilter;
    ZhFilterPreferences.applyFilter = function(widget, p, reloadWidget) {
        return ZhFilterPreferences.oldApplyFilter(widget, p, reloadWidget).then(function(widgetPromiseReturn) {
            //handleStripe(widget);
            updateStripe(widget);
        }.bind(this));
    };

    this.onClickDependentField = function(dependentField, parameters) {

        var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

        var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

        if (changedDataSourceFilter) {
            changeDependetFieldFilter(dependentField, dataSourceFilterNew);
        }

        if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
            dependentField.reload();
        }
    };

    this.buildDataSourceFilter = function(dataSourceFilter, parameters) {
        if (!dataSourceFilter) {
            throw "DataSourceFilter is not defined!";
        }

        for (var position in dataSourceFilter) {
            if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
                dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
            }
        }

        return dataSourceFilter;
    };

    var changeDependetFieldFilter = function(dependentField, dataSourceFilterNew) {

        dependentField.dataSourceFilter = dataSourceFilterNew;
        dependentField.dataSource.lazyLoad = false;
        if (dependentField.selectWidget) {
            dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
        }

        if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

            searchListDeleted = true;
            delete dependentField.selectWidget.searchList;

        } else if (dependentField.selectWidget) {

            searchListDeleted = false;

        }

    };

    var compareDataSourceFilters = function(selectWidget, dataSourceFilterNew, parameters) {
        var changedDataSourceFilter = false;
        if (selectWidget) {
            var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
            for (var position = 0; position < dataSourceFilterNew.length; position++) {
                if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
                    if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
                        changedDataSourceFilter = true;
                    }
                }
            }
        } else {
            changedDataSourceFilter = true;
        }
        return changedDataSourceFilter;
    };

    /*
     * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
     * ou produtos correspondentes à máscara utilizada. Através de cada nível
     * da máscara, é gerado o número correspondente à ele.
     *
     * @param string maskName
     * @param int firstLevel
     *
     * @return array
     *
     * Exemplo:
     * InflightorGeneral.getLevelByMask('CDPICTPRAT', 1);
     */
    this.getLevelByMask = function(maskName, firstLevel) {

        firstLevel = toNumber(firstLevel);

        var mask = getLocalVar(maskName);
        var splitMask = mask.split('.');
        var biggestLevel = splitMask.length;

        var level = [];
        var objLevel = {};

        for (var i = 0; i < biggestLevel; i++) {

            objLevel = {
                nivel: i + firstLevel,
                caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
            };

            objLevel.nivel = objLevel.nivel.toString();
            objLevel.caracteres = objLevel.caracteres.toString();

            level.push(objLevel);
        }

        return level;
    };

    var toNumber = function(value) {

        if (!value && value !== 0) {

            throw 'firstLevel is not defined';

        } else if (typeof value !== 'number' && value) {

            value = parseInt(value);

            if (isNaN(parseInt(value))) {

                throw 'Invalid firstLevel';

            } else {

                return value;
            }
        }
        return value;
    };

    this.toggleReadOnlySelect = function(checkboxField, selectField) {

        if (checkboxField.checked) {
            selectField.readOnly = true;
            selectField.validations = {};
            selectField.setValue('');
        } else {
            selectField.readOnly = false;
            selectField.validations = {
                required: true
            };
        }
        templateManager.updateTemplate();
    };

    this.setLoggedUnit = function(widget) {
        widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
        widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
    };

    var currentIdgrupoper;
    var currentCallback;
    var currentParameters;
    var gruposDeOperador = {
        LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
    };

    this.openAuthPopup = function(idgrupoper, message, callback, parameters) {
        if (!idgrupoper) {
            throw 'idgrupoper is not defined';
        }
        MessageService.getMessagesByCode(["product#EMAIL", "product#SENHA", "product#CANCELAR", "product#CONFIRMAR", "product#G_" + idgrupoper, "AUTENTICACAO"]).then(function(mensagens) {
            var fields = [
                {
                    "name": "email",
                    "label": mensagens[0],
                    "class": 12,
                    "isVisible": true,
                    "readOnly": false,
                    "template": "field/text-edit.html",
                    "events": [],
                    "fieldGroup": "AUTH",
                    "validations": {
                        "required": true,
                        "mail": true
                    }
                },
                {
                    "name": "senha",
                    "label": mensagens[1],
                    "class": 12,
                    "isVisible": true,
                    "readOnly": false,
                    "fieldGroup": "AUTH",
                    "template": "field/password-edit.html",
                    "events": [],
                    "validations": {
                        "required": true
                    }
                }
            ];
            var actions = [
                {
                    "name": "cancelar",
                    "label": mensagens[2],
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": mensagens[3],
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "InflightorGeneral.validateUser(args.owner.widget);"
                        }
                    ]
                }
            ];

            initAuthVariables(idgrupoper, callback, parameters);

            if (!message) {
                message = mensagens[4] || '';
            }

            var widgetAuth = metaDataFactory.widgetFactory({
                "id": 0,
                "name": "authWidget",
                "label": mensagens[5],
                "template": "widget/form.html",
                "container": templateManager.container,
                "isVisible": true,
                "popupNoBlock": false,
                "fieldGroups": [
                    {
                        "id": 100000,
                        "name": "AUTH",
                        "opened": true,
                        "collapsible": false,
                        "label": message
                    }
                ],
                "fields": fields,
                "actions": actions,
                "events": []
            });

            ScreenService.openPopup(widgetAuth);
        });

    };

    var initAuthVariables = function(idgrupoper, callback, parameters) {
        currentIdgrupoper = idgrupoper;
        currentCallback = callback || null;
        currentParameters = parameters || [];
    };

    this.validateUser = function(widget) {

        var self = this;

        if (widget.isValid()) {

            var email = widget.currentRow.email;
            var senha = widget.currentRow.senha;

            autenticar(email, senha).then(function(callback) {

                getPermission(callback.dataset.user.cdoperador).then(function(permission) {

                    if (permission) {

                        if (typeof currentCallback == 'function') {

                            ScreenService.closePopup();

                            currentCallback.apply(null, currentParameters);

                        } else {

                            throw 'callback is not a function';

                        }

                    } else {

                        var message = 'Operador não pertence ao grupo responsável pela autorização.';

                        MessageService.showMessageByCode("product#OPER_N_PERTENCE_GRUPO_AUTO");

                    }

                }, function(callback) {

                    this.callbackError(callback);

                }.bind(self));

            }, function(callback) {

                this.callbackError(callback);

            }.bind(self));
        }

    };

    var autenticar = function(email, senha) {

        return restEngine.request('/Autenticar', {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "email",
                    operator: "=",
                    value: email
                },
                {
                    name: "senha",
                    operator: "=",
                    value: senha
                }
            ]
        });

    };

    var getPermission = function(cdoperador) {

        return restEngine.request('/validateUserGroup', {
            "requestType": "Row",
            "row": {
                "idgrupoper": currentIdgrupoper,
                "cdoperador": cdoperador
            }
        }).then(function(callback) {

            return callback.dataset.permission.isValid;
        });

    };

    this.callbackError = function(callback) {

        if (callback.data) {

            ScreenService.showMessage(ScreenService.i18n(callback.data.error));

        } else {

            ScreenService.showMessage(ScreenService.i18n('Timeout'));
        }
    };

    this.comparaPratoProduto = function(pinicial, pfinal, mascara) {
        var mask = {
            type: "fix",
            params: {
                mask: mascara
            }
        };
        pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
        pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
        var nvinicial = pinicial.length;
        var nvfinal = pfinal.length;
        var menornv;
        if (nvinicial > nvfinal) {
            menornv = nvfinal;
        } else {
            menornv = nvinicial;
        }

        var resultado = true;
        var i = 0;

        while (pinicial[i] == pfinal[i] && i < menornv) {
            i++;
        }

        if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
            resultado = false;
        } else {
            resultado = true;
        }

        return resultado;
    };

    var defaultFormatsReport = [
        {
            "name": "pdf",
            "value": "PDF"
        },
        {
            "name": "xls",
            "value": "XLS"
        },
        {
            "name": "rtf",
            "value": "RTF"
        },
        {
            "name": "txt",
            "value": "TXT"
        }
    ];

    var reportCallback;
    var reportParameters;
    var reportWidgetParentName;

    this.openReportPopup = function(widget, callback, parameters, formatsReport) {
        MessageService.getMessagesByCode(["product#TIPO_REL", "product#CANCELAR", "product#CONFIRMAR"]).then(function(mensagens) {
            var reportWidget = {
                "id": 0,
                "name": "reportWidget",
                "label": mensagens[0],
                "template": "widget/form.html",
                "container": {},
                "isVisible": true,
                "popupNoBlock": false,
                "fields": [],
                "actions": [],
                "events": []
            };
            var reportFields = [
                {
                    "id": 99999999,
                    "name": "QR2TYPE",
                    "label": "",
                    "class": 12,
                    "readOnly": false,
                    "isVisible": true,
                    "isVisibleForm": true,
                    "isVisibleGrid": false,
                    "template": "field/radio-group.html",
                    "events": [],
                    "valueField": "value",
                    "descriptionField": "name",
                    "defaultOption": 0,
                    "dataSource": {
                        "data": []
                    }
                }
            ];
            var reportActions = [
                {
                    "name": "cancelar",
                    "label": mensagens[1],
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup();"
                        }
                    ]
                },
                {
                    "name": "confirmar",
                    "label": mensagens[2],
                    "idVisible": true,
                    "hideIcon": true,
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "InflightorGeneral.generateReport(args.owner.widget, args);"
                        }
                    ]
                }
            ];
            reportCallback = callback;
            reportParameters = parameters || [];
            reportWidgetParentName = widget.name;
            reportWidget.container = widget.container;
            reportFields[0].dataSource.data = formatsReport ? formatsReport : defaultFormatsReport;
            reportWidget.fields = reportFields;
            reportWidget.actions = reportActions;

            var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

            ScreenService.openPopup(reportWidgetInitialized);
        });
    };

    this.generateReport = function(widget, args) {

        var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

        reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

        args.row = reportWidgetParentCurrentRow;

        if (typeof reportCallback == 'function') {

            reportCallback.apply(null, reportParameters);
        }

        ScreenService.closePopup();

    };

    this.saveRow = function(args, closeSwipe) {
        args.owner.widget.saveRow().then(function() {
            var currentRow = args.owner.widget.currentRow;
            currentRow.__is_new = false;
            currentRow.changed = false;
            args
                .owner
                .widget
                .reload();
            if (closeSwipe) {
                ScreenService.closeSwipe();
            }
        });
    };

    this.openSelectionOnce = function(name) {
        var filterWidget = templateManager.container.getWidget(name + "_FILTER");
        if (!filterWidget.wasOpened) {
            filterWidget.edit();
            ScreenService.openPopup(filterWidget);
            filterWidget.wasOpened = true;
        }
    };

    this.openSelection = function(name) {
        var filterWidget = templateManager.container.getWidget(name + "_FILTER");
        filterWidget.edit();
        ScreenService.openPopup(filterWidget);
    };

    this.getContainerName = function() {
        return templateManager.container.name;
    };

    this.backOrCancel = function(args) {
        var deferred = ZHPromise.defer();
        if (args.row.__is_new) {
            args.owner.widget.list(true);
            args.owner.widget.dataSource.data.pop();
            args.owner.widget.currentRow = {};
            deferred.resolve();
        } else if (args.owner.widget.mode === 'edit') {
            args.owner.widget.view();
            for (var i in args.owner.widget.backupRow) {
                args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
            }
            deferred.resolve();
            return deferred.promise;
        }
        ScreenService.goBack();
        return deferred.promise;
    };

    this.edit = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
    };

    this.modEdit = function(widget) {
        parent.edit(widget);
        var widgetParent = widget.widgets;
        if (widgetParent.length)
            parent.setVisibleforTabbed(widgetParent, false, widget);
        // widget.template = widget.formTemplate = widget.templateToForm;
    };

    this.cancelModEdit = function(args, widgetParent) {
        parent.backOrCancel(args).then(function(success) {
            var widget = args.owner.widget;
            var widgetParent = widget.widgets;
            if (widgetParent.length)
                parent.setVisibleforTabbed(widgetParent, true);
            // widget.template = widget.formTemplate = widget.templateToView;
        });
    };

    this.setVisibleforTabbed = function(widgets, isVisible, exception) {
        widgets.forEach(function(widget) {
            if (exception) {
                if (exception.name != widget.name)
                    widget.isVisible = isVisible;
            } else {
                widget.isVisible = isVisible;
            }
        });
    };

    this.setContextVar = function(name, containerName, value) {
        localStorageEngine.setLocalVar(containerName + '_' + name, value);
    };

    this.getContextVar = function(name, containerName) {
        return localStorageEngine.getLocalVar(containerName + '_' + name);
    };

    var _getWidget = function(widgetName) {
        var isPopup = arguments[1] || null;
        if (!!templateManager.container) {
            var widget;
            if (!isPopup) {
                widget = templateManager.container.getWidget(widgetName);
            }
            // else {
            //     widget = templateManager.container.getWidget(widgetName).__filterPopup;
            // }
            if (widget) {
                return widget;
            }
        }
        return null;
    };

    var _getField = function(name, widgetName) {
        var widget = _getWidget(widgetName);
        var field = widget.getField(name);
        if (field) {
            return field;
        }
        return null;
    };

    var _getAction = function(name, widgetName) {
        var widget = _getWidget(widgetName);
        var action = widget.getAction(name);
        if (action) {
            return action;
        }
        return null;
    };

    this.getFieldValue = function(name, widgetName) {
        var isPopup = arguments[2] || null;
        var widget = _getWidget(widgetName, isPopup);
        if (widget && widget.currentRow) {
            return widget.currentRow[name];
        } else {
            return "";
        }
    };

    this.setEnableField = function(name, widgetName) {
        _getField(name, widgetName).readOnly = false;
    };

    this.setDisableField = function(name, widgetName) {
        _getField(name, widgetName).readOnly = true;
    };

    this.setVisibleField = function(name, widgetName) {
        _getField(name, widgetName).isVisible = true;
    };

    this.setInvisibleField = function(name, widgetName) {
        _getField(name, widgetName).isVisible = false;
    };

    this.setEnableAction = function(name, widgetName) {
        _getAction(name, widgetName).readOnly = false;
    };

    this.setDisableAction = function(name, widgetName) {
        _getAction(name, widgetName).readOnly = true;
    };

    this.setVisibleAction = function(name, widgetName) {
        _getAction(name, widgetName).isVisible = true;
    };

    this.setInvisibleAction = function(name, widgetName) {
        _getAction(name, widgetName).isVisible = false;
    };

    this.now = function() {
        return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
    };

    this.today = function() {
        return moment(new Date()).format('DD/MM/YYYY');
    };

    this.openReport = function(route, reportName, params) {
        params.__report_name = reportName;
        return restEngine.post(route, {
            "requestType": "Row",
            "row": params
        }, null, null, null, null, 3000000).then(function(response) {
            if (response && response.messages) {
                var report = response.messages.pop();
                var opened = window.open(report.message);
                if (!opened) {
                    MessageService.showMessageByCode("product#POPUP_BLOQ");
                }
            }
        }).catch(function(response) {
            if (response.data && response.data.error) {
                ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
            }
        });
    };

    // CRUD
    /**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
    this.saveCrud = function(widget, afterValid, prepareWidgetAfterSave = true) {
        var deferred = ZHPromise.defer();
        if (widget.isValid()) {
            if (afterValid && angular.isObject(afterValid)) {
                if (angular.isFunction(afterValid.callback))
                    afterValid.callback.apply(null, afterValid.params);
            }

            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                parent.applyParamsNewCodeToRow(widget);
            }

            var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
            if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
                setDuplicationValidateParamsToCurrentRow(widget);
            }

            widget.saveRow().then(function(success) {
                widget.activate();
                if (widget.currentRow.__is_new && prepareWidgetAfterSave) {
                    widget.list(true);
                    ScreenService.goBack();
                    widget.reload().then(function() {
                        deferred.resolve();
                    });
                } else if (prepareWidgetAfterSave) {
                    widget.computeDescriptionOrder();
                    widget.view();
                    deferred.resolve();
                }
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
                deferred.reject(errorMessage);
            });
        } else {
            // @todo create messages class
            deferred.reject("Preencha todos os campos corretamente!");
        }

        return deferred.promise;
    };

    this.applyParamsNewCodeToRow = function(widget) {
        widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
        widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
        widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
        widget.currentRow.__new_code_interval = widget.paramsNewCode.__new_code_interval || 1;
        widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
        widget.currentRow.__new_code_column_size = widget.paramsNewCode.__new_code_column_size || 10;
        widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;

        var keyToConcat = widget.paramsNewCode.__key_to_concat;
        if (keyToConcat) {
            keyToConcat = Array.isArray(keyToConcat) ? keyToConcat : [keyToConcat];
            for (var i = 0; i < keyToConcat.length; i++) {
                var keyItem = keyToConcat[i];
                var keyValue = widget.currentRow[keyItem];

                if (typeof(keyValue) !== 'undefined' && keyValue !== null) {
                    widget.currentRow.__new_code_pk = '' + widget.currentRow.__new_code_pk + keyValue;
                }
            }
        }
    };

    /* setDuplicationValidateParamsToCurrentRow
     *
     * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
     * cado, antes de salvar, se os valores dos campos a serem validados já exis-
     * tem no banco.
     *
     * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
     * do campo que sera exibida na resposta em caso de duplicacao.
     *
     * Exemplo:
     *      "fieldsToValidateDuplication": [
     *        {
     *          "name": "NMCAMPO",
     *          "description": "campo"
     *        }
     *      ]
     */
    var setDuplicationValidateParamsToCurrentRow = function(widget) {
        try {
            var fieldsToValidate = widget.fieldsToValidateDuplication;
            var validationArray = [];

            checkValidateFieldsArray(fieldsToValidate);

            for (var index in fieldsToValidate) {
                var field = fieldsToValidate[index];

                if (!field.description) {
                    field.description = field.name;
                    var warningMsg = "Missing description for " + field.name +
                        " field in 'fieldsToValidateDuplication' of " + widget.name;
                    console.warn(warningMsg);
                }
                field.description = field.description.toLowerCase();

                validationArray.push(field);
            }

            if (validationArray.length) {
                widget.currentRow.__fieldsToValidateDuplication = validationArray;
            }
        } catch (e) {
            ScreenService.showMessage(e);
        }
    };

    var checkValidateFieldsArray = function(arrayToCheck) {
        for (var index in arrayToCheck) {
            var objectToCheck = arrayToCheck[index];

            if (typeof(objectToCheck) == "object") {
                if (!objectToCheck.name) {
                    throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
                }
            } else {
                throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
            }
        }
    };

    this.deleteCrud = function(widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode("product#DEL_REGISTER").then(function(success) {
            widget.removeRow().then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.activate();
                    deferred.resolve();
                });
            }, function(error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode("product#REGISTRO_POSSUI_FILHOS");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

    this.deleteCrudCheckedRows = function(widget) {
        var deferred = ZHPromise.defer();
        MessageService.confirmMessageByCode("product#DEL_REGISTERS").then(function(success) {
            widget.removeCheckedRows().then(function() {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function(error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    MessageService.showMessageByCode("product#REGISTRO_POSSUI_FILHOS");
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

    this.justInteger = function(selector, retryCount) {
        if (!retryCount || retryCount > 5)
            retryCount = 5;
        if (!selector) {
            throw {
                "message": "Selector is not defined!"
            };
        }
        setTimeout(function() {
            var elem = $(selector);
            if (elem[0]) {
                var length = selector.split(',').length;
                for (var i = 0; i < length; i++) {
                    if (elem.hasOwnProperty(i)) {
                        elem[i].onkeypress = avoidNonInteger;
                        elem[i].onpaste = preventEvent;
                    }
                }
            } else {
                if (!retryCount) {
                    console.error("Selector not found!");
                } else {
                    retryCount--;
                    this.justInteger(selector, retryCount);
                }
            }
        }.bind(this), 500);
    };

    function avoidNonInteger(e) {
        var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
        var numberCode = e.which;
        if (integerNumbers.indexOf(numberCode) < 0) {
            preventEvent(e);
        }
    }

    function preventEvent(e) {
        e.preventDefault();
    }

    function padLeft(value, length) {
        return (value.toString().length < length) ? padLeft("0" + value, length) : value;
    }

    function padRight(value, length) {
        return (value.toString().length < length) ? padRight(value + "0", length) : value;
    }

    this.getCurrentDate = function(format) {
        return moment().format(format);
    };

    this.setDateAttrToVarcharOnCurrentRow = function(currentRow, attrName) {
        this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
    };

    this.setAttrOnCurrentRow = function(currentRow, attrName, value) {
        currentRow[attrName] = value;
    };

    this.setFloatAttrOnCurrentRow = function(currentRow, attrName) {
        this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
    };

    this.valueToFloat = function(value) {
        if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
            if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
                return parseFloat(value);
            } else if (value.indexOf(',') > value.indexOf('.')) {
                return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
            } else if (value.indexOf(',') < value.indexOf('.')) {
                return parseFloat(value.replace(/\,/g, ''));
            }
        } else {
            return value;
        }
    };

    this.toLocalDate = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        $.currency.setMask(params, element);
    };

    this.toLocalDateFormat = function(value, params) {
        if (value) {
            var valueDate = moment(value, params.format);
            value = valueDate.format(params.toFormat);
        }

        return value;
    };

    // maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

    var maskFunction = function(element, viewValue, params, callBack) {
        // do something
    };

    var maskFormatFunction = function(value, params) {
        if (value) {
            var length = value.length;
            if (length === 11)
                value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
            else if (length === 14)
                value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
            return value;
        }
    };

    // maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

    /*
     * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
     * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
     * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
     */

    var constAllWidgets;

    this.setVisibilityByProjectID = function(widgets) {
        if (widgets.length > 1) {
            widgets.forEach(function(childWidget) {
                parent.setVisibilityByProjectID(childWidget.widgets);
                if (childWidget.projects) {
                    childWidget.projects.forEach(function(proj) {
                        childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
                    });
                }
            });

        }

    };

    this.validateFilterPopup = function(args) {
        // var filterPopup = args.owner.__filterPopup;
        // if (!filterPopup.isValid())
        //     args.e.preventDefault();
        console.warn('Retirar chamada da função. Correção feita na atualização do Preferences 3.6.0');
        return true;
    };

    var timeFormats = {
        "hh": isValidHour,
        "mm": isValidMinutesOrSeconds,
        "ss": isValidMinutesOrSeconds,
        "hh:mm": isValidHourAndMinutes,
        "mm:ss": isValidMinutesAndSeconds,
        "hh:mm:ss": isValidTime,
    };

    function isValidHour(hour) {
        if (parseInt(hour) > 23 || parseInt(hour) < 0)
            return false;
        else
            return true;
    }

    function isValidMinutesOrSeconds(minuteOrSecond) {
        if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
            return false;
        else
            return true;
    }

    function isValidHourAndMinutes(hourAndMinute) {
        if (hourAndMinute) {
            var timeArray = hourAndMinute.split(':');
            return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
        }
        return true;
    }

    function isValidMinutesAndSeconds(minutesAndSeconds) {
        if (minutesAndSeconds) {
            var timeArray = minutesAndSeconds.split(':');
            return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
        }
        return true;
    }

    function isValidTime(time) {
        if (time) {
            var timeArray = time.split(':');
            return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
        }
        return true;
    }

    var validTime = function(value, format) {
        var validFunction;
        if (!format)
            throw 'format is not defined';
        if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
            validFunction = timeFormats[format.toLowerCase()];
        else
            throw 'time format is not registered';
        return {
            "valid": validFunction(value),
            "message": ScreenService.i18n("Is not a valid time")
        };
    };

    this.registerTimeFormat = function(format, formatFunction) {
        if (typeof format !== 'string')
            throw 'time format must be an string';
        if (timeFormats.hasOwnProperty(format.toLowerCase()))
            throw 'time format already registered';
        timeFormats[format] = formatFunction;
    };

    // ValidationEngine.registerValidation('isValidTime', validTime);

    /*
     * Função que preenche um valor com um único caractere até um certo tamanho.
     * Faz zerofill pela esquerda por default.
     *
     * @param string value
     * @param int length
     * @param boolean toRight
     * @param char filler
     *
     * @return string
     *
     * Exemplos:
     * InflightorGeneral.fill('123', 10);            //'0000000123'
     * InflightorGeneral.fill('123', 10, true);      //'1230000000'
     * InflightorGeneral.fill('123', 10, null, '_'); //'_______123'
     */
    this.fill = function(value, length, toRight, filler) {
        if (value) {
            value = value.toString();

            if (filler) {
                filler = filler.toString()[0];
            } else filler = '0';

            if (toRight)
                while (value.length + filler.length <= length) value = value + filler;
            else
                while (value.length + filler.length <= length) value = filler + value;
        }
        return value;
    };

    /*
     * Função que faz a validação de CNPJ.
     *
     * @param string CNPJ
     *
     * @return boolean
     */
    function validCNPJ(CNPJ) {
        if (CNPJ === undefined || CNPJ === null)
            return false;

        CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

        if (CNPJ.length != 14)
            return false;

        if (CNPJ == "00000000000000" ||
            CNPJ == "11111111111111" ||
            CNPJ == "22222222222222" ||
            CNPJ == "33333333333333" ||
            CNPJ == "44444444444444" ||
            CNPJ == "55555555555555" ||
            CNPJ == "66666666666666" ||
            CNPJ == "77777777777777" ||
            CNPJ == "88888888888888" ||
            CNPJ == "99999999999999")
            return false;

        var tamanho = CNPJ.length - 2;
        var numeros = CNPJ.substring(0, tamanho);
        var digitos = CNPJ.substring(tamanho);
        var soma = 0;
        var pos = tamanho - 7;

        for (var i = tamanho; i >= 1; i--) {
            soma += numeros.charAt(tamanho - i) * pos--;
            if (pos < 2)
                pos = 9;
        }

        var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
        if (resultado != digitos.charAt(0)) //debugger;
            return false; //'resultado' é o 1º digito de validação do CNPJ

        tamanho = tamanho + 1;
        numeros = CNPJ.substring(0, tamanho);
        soma = 0;
        pos = tamanho - 7;

        for (i = tamanho; i >= 1; i--) {
            soma += numeros.charAt(tamanho - i) * pos--;
            if (pos < 2)
                pos = 9;
        }

        resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
        if (resultado != digitos.charAt(1)) //debugger;
            return false; //'resultado' é o 2º digito de validação do CNPJ

        return true;
    }

    this.validateCNPJ = validCNPJ;

    /*
     * Função que faz a validação de CPF.
     *
     * @param string CPF
     *
     * @return boolean
     */
    function validCPF(CPF) {
        if (CPF === undefined || CPF === null)
            return false;

        CPF = CPF.toString().replace(/[\D]+/g, '');

        if (CPF.length != 11)
            return false;

        if (CPF == "00000000000" ||
            CPF == "11111111111" ||
            CPF == "22222222222" ||
            CPF == "33333333333" ||
            CPF == "44444444444" ||
            CPF == "55555555555" ||
            CPF == "66666666666" ||
            CPF == "77777777777" ||
            CPF == "88888888888" ||
            CPF == "99999999999")
            return false;

        var soma = 0;
        var resto;

        for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
        resto = (soma * 10) % 11;

        if ((resto == 10) || (resto == 11)) resto = 0;
        if (resto != parseInt(CPF.substring(9, 10))) //debugger;
            return false; //'resto' é o 1º dígito de validação do CPF

        soma = 0;
        for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
        resto = (soma * 10) % 11;

        if ((resto == 10) || (resto == 11)) resto = 0;
        if (resto != parseInt(CPF.substring(10, 11))) //debugger;
            return false; //'resto' é o 2º dígito de validação do CPF

        return true;
    }

    this.validateCPF = validCPF;

    /*
     * Função que faz a formatação de uma inscrição,
     * seguindo o padrão de código genérico, CPF, CNPJ e
     * inscrição genérica. Caso o parâmetro "force" for
     * true, o método tenta forçar uma inscrição válida.
     *
     * Retorna um objeto com as seguintes propriedades:
     *  * parameter //string de entrada contendo apenas números
     *  * value     //string formatada conforme o tipo da inscrição
     *  * tipo.cod  //char do código padrão do tipo da inscrição
     *  * tipo.desc //string de descrição do tipo da inscrição
     *  * valid     //boolean
     *
     * @param string inscricao
     * @param boolean force
     *
     * @return object
     */
    this.inscricaoFormat = function(inscricao, force) {
        if (inscricao === undefined || inscricao === null || inscricao === '')
            return {
                'parameter': '',
                'value': '',
                'tipo': {
                    'cod': null,
                    'desc': null
                },
                'valid': false
            };

        if (force === undefined || force === null)
            force = false;
        inscricao = inscricao.toString().replace(/[\D]+/g, '');

        var obj = {};
        obj.tipo = {};

        if (inscricao.length <= 5) {
            obj.tipo.cod = 'O';
            obj.tipo.desc = 'Código Genérico';
            obj.valid = true;
            obj.value = this.fill(inscricao, 5);
        } else {
            if (inscricao.length == 11) {
                obj.tipo.cod = 'F';
                obj.tipo.desc = 'Pessoa Física';
                obj.valid = this.validateCPF(inscricao);
                obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
            } else if (inscricao.length == 14) {
                obj.tipo.cod = 'J';
                obj.tipo.desc = 'Pessoa Jurídica';
                obj.valid = this.validateCNPJ(inscricao);
                obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
            } else force = true;

            if (force && obj.valid !== true) {
                obj.tipo.cod = 'I';
                obj.tipo.desc = 'Inscrição Internacional';

                if (inscricao.length <= 20) {
                    obj.valid = true;
                    obj.value = this.fill(inscricao, 20);
                } else {
                    obj.valid = false;
                    obj.value = inscricao.substr(0, 20);
                }
            }
        }

        obj.parameter = inscricao;
        return obj;
    };

    var CPFValidation = function(value, validationAttrs) {
        return {
            "valid": validCPF(value),
            "message": ScreenService.i18n('CPF Inválido')
        };
    };

    // ValidationEngine.registerValidation('CPF', CPFValidation);

    var CNPJValidation = function(value, validationAttrs) {
        return {
            "valid": validCNPJ(value),
            "message": ScreenService.i18n('CNPJ Inválido')
        };
    };

    // ValidationEngine.registerValidation('CNPJ', CNPJValidation);

    this.callRestEngine = function(dataSource, requestType, requestValue) {
        var defer = ZHPromise.defer();
        var params = {};

        switch (requestType) {
            case "Row":
                params = {
                    "requestType": "Row",
                    "row": requestValue
                };
                break;

            case "DataSet":
                params = {
                    "requestType": "DataSet",
                    "dataset": requestValue
                };
                break;

            case "FilterData":
                params = {
                    "requestType": "FilterData",
                    "filter": requestValue
                };
                break;

            default:
                return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
        }

        return restEngine.request(dataSource, params);
    };

    /*
     * Função que muda a acessibilidade de uma ou mais
     * widgets.
     *
     * @param array(objects) widgets
     * @param boolean enabled
     *
     * Exemplos:
     * InflightorGeneral.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
     * InflightorGeneral.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
     */
    this.changeWidgetAcessibility = function(widgets, enabled) {
        widgets.forEach(function(widget) {
            widget.enabled = enabled;
        });

        templateManager.updateTemplate();
    };

    /*
     * Função que altera o template de um form
     * entre edição e visualização. Retorna um boolean
     * que indica se a widget era do tipo form e a
     * alteração foi realizada.
     *
     * @param object widget
     *
     * @return boolean
     */
    this.formWidgetSwitchTemplate = function(widget, newMethod = false) {
        if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

        if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

        if (newMethod) {
            if (widget.mode == "view") widget.template = widget.templateToView;
            else if (widget.mode == "edit") widget.template = widget.templateToForm;
        } else {
            if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
            else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;
        }

        templateManager.updateTemplate();

        return true;
    };

    /*
     * Função que coloca uma widget em modo de edição,
     * bloqueando as demais widgets. Retorna um boolean
     * que indica se a alteração de modo foi executada.
     *
     * @param object widget
     *
     * @return boolean
     */
    this.widgetEdit = function(widget) {
        if (!widget.editing) {
            //Trata a edição de uma widget filha do tipo form
            if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
                var widgetsToDeactivate = widget.parent.widgets;

                this.changeWidgetAcessibility(
                    widgetsToDeactivate.filter(function(subwidget) {
                        return subwidget.name != widget.name;
                    }).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
                    false);
            } else //trata a edição dos demais casos
                this.changeWidgetAcessibility(widget.widgets, false);

            //propriedades a serem ignoradas na criação do backup
            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

            //faz backup da currentRow para permitir a recuperação dos dados
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
            return true;
        } else return false;
    };

    /*
     * Procedimento que cancela o formulário de uma
     * widget, seja este de inserção ou de edição, e
     * desbloqueia as demais widgets.
     *
     * @param object widget
     */
    this.widgetCancelForm = function(widget) {
        //trata o cancelamento de uma widget filha do tipo form
        if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
            var widgets = widget.parent.widgets;

            this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
        } else //trata o cancelamento dos demais casos
            this.changeWidgetAcessibility(widget.widgets, true);

        //caso a widget estiver em modo de inserção, remove a row e fecha o swipe
        if (widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if (indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }

            widget.list();

            ScreenService.closeSwipe().then(function() {
                if (widget.parent && widget.parent.template == 'widget/tabbedWidget.html' && widget.parent.parent)
                    widget.parent.parent.activate();
            });
        } else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(widget.backupRow).forEach(function(key) {
                widget.currentRow[key] = widget.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            widget.view();
        }
    };

    /*
     * Procedimento que fecha o swipe e volta ao grid da
     * própria widget. Provavelmente usado na action de
     * 'voltar' do back_view.
     *
     * @param object widget
     */
    this.widgetGoBackToList = function(widget) {
        ScreenService.closeSwipe();
        widget.list();
    };

    /*
     * Procedimento que fecha o swipe e volta ao grid da
     * widget parent. Provavelmente usado na action de
     * 'voltar' do back_list.
     *
     * @param object widget
     */
    this.widgetGoBackToParent = function(widget) {
        ScreenService.closeSwipe().then(function() {
            widget.parent.list();
            widget.parent.activate();
        });
    };

    /*
     * Procedimento que habilita inserção em uma widget,
     * bloqueando as demais widgets.
     *
     * @param object widget
     */
    this.widgetNewRow = function(widget) {
        //  this.changeWidgetAcessibility(widget.widgets, false);

        widget.newRow();
        widget.edit();
        //Trata a edição de uma widget filha do tipo form
        if (this.formWidgetSwitchTemplate(widget, true) && widget.parent) {
            var widgetsToDeactivate = widget.parent.widgets;

            this.changeWidgetAcessibility(
                widgetsToDeactivate.filter(function(subwidget) {
                    return subwidget.name != widget.name;
                }).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
                false);
        } else //trata a edição dos demais casos
            this.changeWidgetAcessibility(widget.widgets, false);
    };

    /*
     * Função que salva os dados de uma widget e retorna
     * para o modo adequado de visualização ou listagem.
     * Faz exibição de mensagens de aviso para o usuário
     * conforme padronização e desbloqueia as demais
     * widgets.
     *
     * Basicamente, um saveCrud adaptado ao padrão.
     *
     * @param object widget
     *
     * @return promise
     */
    this.widgetSave = function(widget, afterValid) {
        var deferred = ZHPromise.defer();
        widget.currentRow = widget.currentRow || {};

        if (widget.isValid()) {
            if (afterValid && angular.isObject(afterValid)) {
                if (angular.isFunction(afterValid.callback))
                    afterValid.callback.apply(null, afterValid.params);
            }

            if (!widget.currentRow.__is_new) {
                widget.currentRow.__is_new = false;
            }

            var paramsNewCode = widget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                parent.applyParamsNewCodeToRow(widget);
            }

            var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
            if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
                setDuplicationValidateParamsToCurrentRow(widget);
            }

            widget.currentRow.changed = true;

            widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                delete widget.backupRow;

                if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
                    widget.view();

                    var widgets = widget.parent.widgets;
                    this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
                } else {
                    this.changeWidgetAcessibility(widget.widgets, true);
                }

                if (widget.currentRow.__is_new) {
                    widget.list(true);
                    ScreenService.goBack();
                } else {
                    widget.computeDescriptionOrder();
                    widget.view();
                    deferred.resolve(response);
                }

                widget.activate();
                widget.reload();

                deferred.resolve(response);
            }.bind(this), function(error) {
                parent.treatAndShowError(error);

                var errorMessage = (error.data) ? 'Timeout' : error.data.error;
                deferred.reject(errorMessage);
            });
        } else {
            deferred.reject("Preencha todos os campos corretamente!");
        }

        return deferred;
    };

    /*
     * Procedimento que recarrega os dados da widget.
     *
     * @param object widget
     */
    this.widgetReload = function(widget, additionalFilters, shouldRefreshCurrentRow) {
        var deferred = ZHPromise.defer();

        widget.reload(additionalFilters, shouldRefreshCurrentRow).then(function() {
            if (widget.dataSource && widget.dataSource.clearCheckedRows) {
                widget.dataSource.clearCheckedRows();
            }
            deferred.resolve();
        }, function(error) {
            parent.treatAndShowError(error);
            deferred.reject();
        });

        return deferred;
    };

    /*
     * Função para validar containers abertos a partir de outros containers.
     * Se o container atual não foi aberto a partir do container pai, a
     * callback é executada. Se nenhuma callback for passada nos parâmetros,
     * o container pai é aberto.
     *
     * @param: object {parentContainerName, callback, scope, params}
     *
     * Exemplos:
     * InflightorGeneral.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
     * InflightorGeneral.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
     */
    this.checkParentContainer = function(containerProperties) {
        containers = templateManager.containers;
        if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
            if (angular.isFunction(containerProperties.callback)) {
                containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
            } else {
                ScreenService.openWindow(containerProperties.parentContainerName);
            }
        }
    };

    /* Função abre um popup com um Text Area a título de mostrar erros em lista.
     * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
     * @params: widget, dataset, popupName
     */
    this.showErrorMessages = function(widget, dataset, popupName) {
        MessageService.getMessagesByCode(["product#MENSAGENS", "product#CANCELAR", "product#EXPORTAR"]).then(function(mensagens) {
            var messagesWidget = {
                "id": 0,
                "name": "messagesWidget",
                "label": mensagens[0],
                "messagesDataset": "",
                "template": "widget/form.html",
                "container": {},
                "isVisible": true,
                "popupNoBlock": false,
                "fields": [],
                "actions": [],
                "events": []
            };

            var messagesFields = [{
                "label": "",
                "name": "CONTENT",
                "readOnly": true,
                "template": "field/textarea.html",
                "textRows": 14,
                "isVisible": true,
                "class": 12,
                "descriptionOrder": true,
                "showOnView": true,
                "showOnForm": true,
                "showOnList": true
            }];

            var messagesActions = [{
                "name": "cancelar",
                "label": mensagens[1],
                "idVisible": true,
                "hideIcon": true,
                "showAsAction": "back",
                "events": [{
                    "name": "ActionEvent",
                    "code": "ScreenService.closePopup();"
                }]
            }, {
                "name": "exportar",
                "label": mensagens[2],
                "idVisible": true,
                "icon": "export",
                "showAsAction": "always",
                "events": [{
                    "name": "ActionEvent",
                    "code": "InflightorGeneral.generateFileByDataSet(args.owner.widget.messagesDataset);"
                }]
            }];

            messagesWidget.container = widget.container;
            messagesWidget.fields = messagesFields;
            messagesWidget.actions = messagesActions;

            var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
            var content = '';
            if (popupName !== undefined)
                messagesWidgetInitialized.label = popupName;
            messagesWidgetInitialized.messagesDataset = dataset;
            dataset.forEach(function(message) {
                content += message + '\n';
            });
            messagesWidgetInitialized.getField('CONTENT').setValue(content);
            ScreenService.openPopup(messagesWidgetInitialized);
        });
    };

    /* Gera arquivo básico baseado em linhas de dataSet.
     * @params: dataset, object {fileName}
     * !!! fileExtension: por enquanto somente TXT;
     * Exemplo:
     * InflightorGeneral.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
     */

    this.generateFileByDataSet = function(dataset, fileProperties) {
        var content = "";
        var name = 'content';
        var extension = 'text/plain';

        if (fileProperties !== undefined) {
            if (fileProperties.fileName !== undefined)
                name = fileProperties.fileName;
        }

        dataset.forEach(function(line) {
            content += line + "\r\n";
        });

        var element = document.createElement('a');
        element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
        element.setAttribute('download', name);

        element.style.display = 'none';
        document.body.appendChild(element);

        element.click();

        document.body.removeChild(element);
    };

    /**
     * Este método verifica se a widget foi filtrada usando o zh-grid-filter
     * @param Widget widget
     * @return boolean
     */
    this.isWidgetFiltered = function(widget) {
        var isFiltered = true;
        var currentFilter = widget.appliedFilter.filters;
        var requiredFields = widget.fields.filter(function(field) {
            return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
        });
        if (requiredFields.length && Array.isArray(currentFilter)) {
            isFiltered = false;
        }
        return isFiltered;
    };

    function isRequiredFieldInFilter(field) {
        return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
    }

    function isRequiredFieldWithoutFilterProperties(field) {
        return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
    }

    /**
     * Este método valida a inserção de registros na widget
     * @param Widget widget
     */
    this.newRow = function(widget) {
        if (this.isWidgetFiltered(widget)) {
            widget.newRow();
        } else {
            MessageService.showMessageByCode("product#FILTER_BEFORE_ADD");
        }
    };

    this.applyFilterFromRow = function(dataSourceFilter, row) {
        if (dataSourceFilter && Array.isArray(dataSourceFilter)) {
            dataSourceFilter.forEach(function(filter) {
                var filterName = filter.name;
                if (typeof row[filterName] != 'undefined') {
                    filter.value = row[filterName];
                }
            });
        }
    };

    this.filterChildWidgets = function(parentWidget, filterRow) {
        filterRow = filterRow ? filterRow : parentWidget.currentRow;

        parentWidget.widgets.forEach(function(childWidget) {
            var childFilter = childWidget.dataSourceFilter;
            parent.applyFilterFromRow(childFilter, filterRow);
            parent.widgetReload(childWidget);
        });
    };

    this.openChildSwipe = function(widget, isTabbed) {
        var parentWidget = isTabbed ? widget.widgets[0] : widget;
        parent.filterChildWidgets(parentWidget, widget.currentRow);
        ScreenService.openSwipe(widget.widgets[0]);
    };

    this.doBackAction = function(widget, mode = "swipe") {
        var closeMethod = (mode == "popup") ? ScreenService.closePopup : ScreenService.closeSwipe;

        var isView = (widget.mode == 'view');
        var isMasterDetail = (widget.template == 'widget/master_detail/grid.html');

        var parentWidget = widget.getParent();
        var isParentTabbed = (parentWidget && parentWidget.template == 'widget/tabbedWidget.html');
        var trueParent = isParentTabbed ? parentWidget.parent : parentWidget;

        closeMethod();

        if (isView && isMasterDetail) {
            widget.list();
            widget.activate();
        } else if (trueParent) {
            trueParent.activate();
        }
    };

    this.sortArrayByProperty = function(array, propertiesArray = []) {
        propertiesArray = Util.isArray(propertiesArray) ? propertiesArray : [propertiesArray];
        return array.sort(function(a, b) {
            var index = 0;
            for (var i = 0; i < propertiesArray.length; i++) {
                var property = propertiesArray[i];
                if (a[property] != b[property]) {
                    return +(a[property] > b[property]) || -1;
                }
            }
            return 0;
        });
    };

    this.openLinkPopup = function(widget) {
        try {
            var linkParams = widget.linkParams;

            var keyRow = getLinkReferenceRow(widget, linkParams.keyRow);
            var filterRow = getLinkReferenceRow(widget, linkParams.filterRow);

            var popupFields = buildLinkPopupFields(linkParams.popupFields, filterRow);
            var popupActions = buildLinkPopupActions();

            var popupWidget = buildLinkPopupWidget(widget, popupFields, popupActions);

            if (linkParams.paramsNewCode) {
                popupWidget.paramsNewCode = linkParams.paramsNewCode;
            }

            popupWidget.newRow();

            linkParams.linkKeys.forEach(function(linkKey) {
                popupWidget.currentRow[linkKey] = keyRow[linkKey];
            });

            ScreenService.openPopup(popupWidget).then(function() {
                popupWidget.edit();
            });
        } catch (e) {
            console.error(e);
        }
    };

    this.saveFromLinkPopup = function(popupWidget) {
        if (popupWidget.isValid() && linkPopupIsValid(popupWidget)) {
            var saveRoute = popupWidget.saveRoute;
            var linkValues = popupWidget.linkValues;
            var selectMode = popupWidget.selectMode;

            var currentRow = popupWidget.currentRow;

            var paramsNewCode = popupWidget.paramsNewCode;
            if (paramsNewCode && paramsNewCode.__is_new_code) {
                parent.applyParamsNewCodeToRow(popupWidget);
            }

            var buildDataSet = function() {
                var dataSet = [];
                //TODO generic function for multidimensions links
                var linkValue = linkValues[0];
                currentRow[linkValue].forEach(function(linkValueElm) {
                    var newRow = angular.copy(currentRow);
                    newRow[linkValue] = linkValueElm;
                    dataSet.push(newRow);
                });

                return dataSet;
            };

            var requestType = (selectMode == 'multiple') ? 'DataSet' : 'Row';
            var requestData = (selectMode == 'multiple') ? buildDataSet(popupWidget) : popupWidget.currentRow;

            if (requestData) {
                parent.callRestEngine(saveRoute, requestType, requestData).then(
                    function() {
                        ScreenService.closePopup();

                        popupWidget.parent.reload();
                        popupWidget.parent.activate();

                        if (popupWidget.saveCallback) {
                            var widget = popupWidget.parent;
                            eval(popupWidget.saveCallback);
                        }
                    },
                    parent.treatAndShowError
                );
            }
        }
    };

    var getLinkReferenceRow = function(widget, referenceRow) {
        var referenceWidget = {
            currentRow: {}
        };

        switch (referenceRow) {
            case 'self':
                if (widget.currentRow) {
                    referenceWidget = widget;
                }
                break;
            case 'parent':
                if (widget.getParent() && widget.getParent().currentRow) {
                    referenceWidget = widget.getParent();
                }
                break;
            case 'parent-parent':
                if (widget.getParent() && widget.getParent().getParent() && widget.getParent().getParent().currentRow) {
                    referenceWidget = widget.getParent().getParent();
                }
                break;
            default:
                if (typeof(referenceRow) == 'object' && referenceRow.widgetName) {
                    var widgetName = referenceRow.widgetName;

                    referenceWidget = widget.container.getWidget(widgetName) ? widget.container.getWidget(widgetName) : referenceWidget;
                    referenceWidget.currentRow = referenceWidget.currentRow ? referenceWidget.currentRow : {};
                }
                break;
        }

        return referenceWidget.currentRow;
    };

    var buildLinkPopupFields = function(fields, filterRow) {
        fields.forEach(function(field) {
            parent.applyFilterFromRow(field.dataSourceFilter, filterRow);
        });
        return fields;
    };

    var buildLinkPopupActions = function() {
        var saveActionCode = 'InflightorGeneral.saveFromLinkPopup(args.owner.widget);';
        var saveAction = {
            'name': 'save',
            'label': 'Salvar',
            'isVisible': true,
            'showAsAction': 'never',
            'hideIcon': true,
            'events': [
                {
                    'name': 'ActionEvent',
                    'code': saveActionCode
                }
            ]
        };
        var cancelAction = {
            'name': 'cancel',
            'label': 'Cancelar',
            'isVisible': true,
            'showAsAction': 'back',
            'hideIcon': true,
            'events': [
                {
                    'name': 'ActionEvent',
                    'code': 'ScreenService.closePopup();'
                }
            ]
        };

        return [saveAction, cancelAction];
    };

    var buildLinkPopupWidget = function(parentWidget, popupFields, popupActions) {
        var linkParams = parentWidget.linkParams;

        var saveRoute = linkParams.saveRoute;
        var saveCallback = linkParams.saveCallback;
        var linkKeys = linkParams.linkKeys;
        var linkValues = linkParams.linkValues;
        var selectMode = linkParams.selectMode ? linkParams.selectMode : 'single';
        var widgetLabel = linkParams.popupLabel ? linkParams.popupLabel : 'Vincular';

        var popupWidget = {
            'container': parentWidget.container,
            'parent': parentWidget,
            'name': 'linkPopupWidget',
            'label': widgetLabel,
            'isVisible': true,
            'template': 'widget/form.html',
            'formTemplate': 'widget/form.html',
            'showDescriptionOrder': false,
            'saveRoute': saveRoute,
            'saveCallback': saveCallback,
            'selectMode': selectMode,
            'linkKeys': linkKeys,
            'linkValues': linkValues,
            'fields': popupFields,
            'actions': popupActions,
            'events': [],
            'widgets': []
        };

        return metaDataFactory.widgetFactory(popupWidget);
    };

    var linkPopupIsValid = function(widget) {
        var currentRow = widget.currentRow;
        var linkKeys = Array.isArray(widget.linkKeys) ? widget.linkKeys : [widget.linkKeys];

        for (var i = 0; i < linkKeys.length; i++) {
            var key = linkKeys[i];
            if (!currentRow[key]) return false;
        }

        return true;
    };

    this.treatAndShowError = function(error) {
        if (error.data.dataset && error.data.dataset.errorList) {
            var widget = templateManager.container.getActiveWidget();
            parent.showErrorMessages(widget, error.data.dataset.errorList);
        } else {
            let tratada = true;
            var errorMessage = 'ERRO_INESPERADO';
            if (error.data && error.data.error) {
                var errorData = error.data.error;
                var existsInError = function(str) {
                    return errorData.indexOf(str) !== -1;
                };

                if (existsInError('ORA-02292')) {
                    errorMessage = 'REGISTRO_POSSUI_FILHOS';
                } else if (existsInError('ORA-20501')) {
                    var index = errorData.indexOf('ORA-20501: ') + 11;
                    var newStr = errorData.slice(index);
                    if (newStr.indexOf('ORA-') !== -1) {
                        newStr = newStr.slice(0, newStr.indexOf('ORA-'));
                    }
                    errorMessage = newStr;
                    tratada = false;
                } else if (existsInError('ORA-00001')) {
                    errorMessage = 'REGISTRO_JA_EXISTE';
                } else if (existsInError('ORA-01400')) {
                    errorMessage = 'CAMPO_OBRIGATORIO_N_INF';
                } else if (existsInError('ORA-02291')) {
                    errorMessage = 'REGISTRO_PAI_NAO_ENCONTRADO';
                } else if (existsInError('ORA-02290')) {
                    errorMessage = 'DADOS_INVALIDOS';
                    var constraintErrorMessages = getLocalVar('constraintErrorMessages');
                    if (typeof(constraintErrorMessages) == 'object') {
                        for (var constraintName in constraintErrorMessages) {
                            if (existsInError(constraintName)) {
                                errorMessage = constraintErrorMessages[constraintName];
                                tratada = false;
                            }
                        }
                    }
                } else if (existsInError('ORA-12899')) {
                    errorMessage = 'VALOR_MUITO_GRANDE';
                } else {
                    errorMessage = errorData;
                    tratada = false;
                }
            }
            if (tratada) {
                MessageService.showMessageByCode('product#' + errorMessage);
            } else {
                errorMessage = ScreenService.i18n(errorMessage);
                ScreenService.showMessage(errorMessage, 'error');
            }
        }
    };

    this.getValueFromFilter = function(filter, key) {
        for (var i = 0; i < filter.length; i++) {
            if (filter[i].name == key) {
                return filter[i].value;
            }
        }
        return null;
    };

    this.validateTimeField = function(field, format) {
        var isValid = true;
        var value = field.value();

        if (value) {
            isValid = moment(value, format).isValid();
            if (!isValid) {
                field.setValue(null);
                MessageService.showMessageByCode("product#HORA_INVALIDA");
            }
        }

        return isValid;
    };

    this.validateInlineEditable = function(widget) {
        return widget && !widget.editing && event && event.target.className && (event.target.className.indexOf('inline-editable') === -1 && event.target.className.indexOf('editable-content') === -1);
    };

    this.validateInterval = function(widget, intervalStartFieldName, intervalEndFieldName) {
        let intervalStartField = widget.getField(intervalStartFieldName);
        let intervalEndField = widget.getField(intervalEndFieldName);

        if (intervalStartField && intervalEndField) {
            let momentIntervalStart = moment(intervalStartField.value(), intervalStartField.dateFormat || "DD/MM/YYYY");
            let momentIntervalEnd = moment(intervalEndField.value(), intervalEndField.dateFormat || "DD/MM/YYYY");

            if (momentIntervalStart && momentIntervalEnd) {
                if (momentIntervalStart.isValid() && momentIntervalEnd.isValid() && momentIntervalStart.isSameOrBefore(momentIntervalEnd)) {
                    return true;
                } else {
                    MessageService.showMessageByCode("product#INTERVALO_VIG_INV");
                    return false;
                }
            }
        }
        return false;
    };

    this.clearDependentFields = function(parentField) {
        let dependentFieldsNames = parentField.dependentFields || [];
        dependentFieldsNames = Array.isArray(dependentFieldsNames) ? dependentFieldsNames : [dependentFieldsNames];
        dependentFieldsNames.forEach(dependentFieldName => {
            let dependentField = parentField.widget.getField(dependentFieldName);
            dependentField.setValue(null);
            if (dependentField.template.indexOf("inf-select-multiple.html" != -1)) {
                dependentField.setValue(dependentField.valueToAll);
            }
            parent.clearDependentFields(dependentField);
        });
    };

    var trataErroProcedure = function(mensagem) {
        if (mensagem && mensagem.length && mensagem.indexOf('APP-') >= 0) {
            let msg_params = mensagem.split('$$');
            let resultObj = {
                MSGCODE: msg_params[0],
                MSGPARAMS: {}
            };
            for (let i = 1; i < msg_params.length; i++) {
                resultObj.MSGPARAMS[(i - 1).toString()] = msg_params[i];
            }
            return resultObj;
        }
        return false;
    };

    this.mensagemErroProcedure = function(mensagem) {
        let obj = trataErroProcedure(mensagem);
        if (obj) {
            MessageService.showMessageByCode('product#' + obj.MSGCODE, obj.MSGPARAMS);
            return true;
        }

        return false;
    };

    this.mensagensErroProcedure = function(mensagens, titulo) {
        if (Array.isArray(mensagens) && mensagens.length) {
            let mensagensATraduzir = [];
            let parametros = [];
            for (let i = 0; i < mensagens.length; i++) {
                let obj = trataErroProcedure(mensagens[i]);
                if (obj) {
                    mensagensATraduzir.push('product#' + obj.MSGCODE);
                    parametros.push(obj.MSGPARAMS);
                } else {
                    mensagensATraduzir.push(mensagens[i]);
                    parametros.push({});
                }
            }
            let mensagensTraduzidas = angular.copy(mensagensATraduzir);
            MessageService.getMessagesByCode(mensagensTraduzidas).then(function(mensagensTraduzidas) {
                for (let i = 0; i < mensagensTraduzidas.length; i++) {
                    if (mensagensTraduzidas[i]) {
                        mensagensTraduzidas[i] = ScreenService.i18n(mensagensTraduzidas[i], parametros[i]);
                    } else {
                        mensagensTraduzidas[i] = ScreenService.i18n(mensagensATraduzir[i]);
                    }
                }

                parent.showErrorMessages(templateManager.container.getActiveWidget(), mensagensTraduzidas, titulo);
            });
            return true;
        }
        return false;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('InflightorGeneral', InflightorGeneral);
});

// FILE: ./assets/js/services/CRM_Operador_Repository.js
var CRM_Operador_Repository = function(localStorageEngine) {
    this.retornaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        return localStorageEngine.download("product#/crud/operador", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('CRM_Operador_Repository', CRM_Operador_Repository);
});

// FILE: ./assets/js/services/ger00003_unidadeService.js
var UnidadeService = function(restEngine, requestDataSourceEngine) {

    this.alterarEstrutura = function(cdfilial, nrorg) {
        return restEngine.post("product#/changeUserUnit", {
            "requestType": "Row",
            "row": {
                "cdfilial": cdfilial,
                "nrorg": nrorg
            }
        });
    };

    this.updateMongoSession = function(nmfilial, cdfilial) {
        var filter = [{
            name: 'OTHER_SESSION_VALUES',
            value: {
                NMFILIAL: nmfilial,
                CDFILIAL: cdfilial
            },
            operator: '='
        }];
        return requestDataSourceEngine.getDataSource("lib#/lib_updateSession", filter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UnidadeService', UnidadeService);
});

// FILE: ./assets/js/services/Parametros.js
function Parametros(localStorageEngine, restEngine) {

    this.paramgeral = function() {

        return restEngine.post('product#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": [{
                "name": "NRORG",
                "operator": "=",
                "value": localStorageEngine.getLocalVar("NRORG")
            }]
        });

    };

    this.parasupri = function(cdfilial) {

        return restEngine.post('product#/crud/parasupri', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "NRORG",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar("NRORG")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });

    };

    this.parcmpsup = function(cdfilial) {

        return restEngine.post('product#/crud/parcmpsup', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "NRORG",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar("NRORG")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });

    };

    this.paramplaneja = function() {

        return restEngine.post('product#/crud/paramplaneja', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "NRORG",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar("NRORG")
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar("CDFILIAL")
                }
            ]
        });

    };

    this.retornaNivelMaxProduto = function() {
        mascaraProduto = getLocalVar('CDPICTPROD');
        nivelMax = mascaraProduto.split('.').length;

        return nivelMax;
    };

    this.retornaNivelMaxPrato = function() {
        mascaraPrato = getLocalVar('CDPICTPRAT');
        nivelMax = mascaraPrato.split('.').length;

        return nivelMax;
    };

    this.paramfilial = function() {
        var nrorg = arguments[0] || localStorageEngine.getLocalVar("NRORG");
        var cdfilial = arguments[1] || localStorageEngine.getLocalVar("CDFILIAL");
        return restEngine.post('product#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });

    };

    this.retornaDataProcessamento = function(cdfilial, nrorg) {

        return this.paramfilial(nrorg, cdfilial).then(function(callback) {

            var dtprocessa = callback.dataset.paramfilial[0].DTPROCESSA;

            if (dtprocessa) {
                return moment(dtprocessa, 'DD/MM/YYYY HH:mm:ss').format('L');
            } else {
                return moment().format('L');
            }

        });

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Parametros', Parametros);
});

// FILE: ./assets/js/controllers/CRM_Operador_Controller.js
function CRM_Operador_Controller(ScreenService, CRM_Operador_Repository, templateManager, restEngine, localStorageEngine, InflightorGeneral) {
    var self = this;
    self.operadorNewRow = function(widget) {
        self.masterDetailNewRow();
        this.setFilialField(widget, true);

        CRM_Operador_Repository.retornaOperadorLogado().then(function(operator) {
            widget.currentRow.NRORG = operator[0].NRORG;
            widget.currentRow.CDPAIS = "0055";
            widget.currentRow.NMPAIS = "Brasil";
            widget.currentRow.NRORGTRAB = operator[0].NRORG;
            self.blockCdoperadorField(widget, true);

            widget.currentRow.CDCLIENTE = null;
            widget.currentRow.CDEMPRESA = null;
            widget.currentRow.DTULTATUOPER = null;
            widget.currentRow.IDOPERACESSO = null;
            widget.currentRow.NRESTRUTTRAB = null;
            widget.currentRow.IDOPERMASTER = 'N';
            widget.currentRow.NRINSCRICAOPARC = null;
            widget.currentRow.NRORGSELECIONADA = null;
            widget.currentRow.CDFILIALSELECIONADA = null;
            widget.currentRow.NRPERFILSELECIONADO = null;

            restEngine.post('/operador/getOrganizacao', {
                    'requestType': 'Row',
                    'row': widget.currentRow
                },
                undefined, undefined, undefined).then(function(response) {
                widget.currentRow.NMORGANIZACAO = response.dataset.DADOS.NMORGANIZACAO;
            });

        }.bind(this));
    };

    this.setFilialField = function(widget, visible) {
        /*var loggedOperator = getLocalVar(''        var') filiaisField   = widget.getField('CDFILIAL');
        
        if(visible === true && loggedOperator.IDOPERMASTER === "S" && widget.currentRow.__is_new === true){
            filiaisField.showOnForm = true;
            if(widget.currentRow.IDOPERMASTER === "N")
                filiaisField.validations = {required:true};
            else
                filiaisField.validations = {};
        }else {
            filiaisField.showOnForm = false;
            filiaisField.validations = {};
        }*/
    };

    this.aplicaStylesGridSelect = function(fieldSelect) {
        fieldSelect.selectWidget.styles = [
            {
                condition: "row.IDATIVO==='N'",
                cssClass: "coc-inactive"
            }
        ];
        fieldSelect.selectWidget.reload();
    };

    this.changeCase = function(field) {
        var valor = field.value();
        field.setValue(valor.toLowerCase());

        /*if (field.name == 'DSEMAILOPER') {
            field.widget.currentRow.DSEMAILOPER = field.widget.currentRow.DSEMAILOPER.toLowerCase();
            field.reload();
        }

        if (field.name == 'NMACESSO') {
            field.widget.currentRow.NMACESSO = field.widget.currentRow.NMACESSO.toUpperCase();
            field.reload();
        }*/
    };

    this.fillOperador = function(field) {
        var widget = field.widget;
        var cdOper = widget.currentRow.CDOPERADOR;

        cdOper = '000000000000' + cdOper;
        cdOper = cdOper.substr(cdOper.length - 12);
        widget.currentRow.CDOPERADOR = cdOper;
        field.reload();
    };

    self.blockCdoperadorField = function(widget, state) {
        widget.getField('CDOPERADOR').readOnly = state;
    };

    self.perfilNewRow = function(widget) {
        self.masterDetailNewRow();

        widget.getField('NMPERFILACESSO').showOnForm = false;
        var field = widget.getField('NMPERFILACESSO_MULT');
        field.showOnForm = true;
        field.dataSourceFilter[0].value = widget.getParent().currentRow.NRPARCNEGOCIO;
        field.reload();

        widget.currentRow.NRPARCNEGOCIO = widget.getParent().currentRow.NRPARCNEGOCIO;
    };

    self.filialNewRow = function(widget) {
        self.masterDetailNewRow();

        widget.getField('NMFILIAL').showOnForm = false;
        var field = widget.getField('NMFILIAL_MULT');
        field.showOnForm = true;
        field.dataSourceFilter[0].value = widget.getParent().currentRow.CDOPERADOR;
        field.reload();

        CRM_Operador_Repository.retornaOperadorLogado().then(function(operator) {
            widget.currentRow.CDOPERADOR = widget.getParent().currentRow.CDOPERADOR;
            widget.currentRow.NRORGFILIAL = operator[0].NRORG;
            widget.currentRow.IDEXIBAVISO = null;
            widget.currentRow.DTULTATUFOPE = null;
        });
    };

    self.widgetEdit = function(widget, field) {
        widget.getField(field).showOnForm = true;
        widget.getField(field + '_MULT').showOnForm = false;
        widget.getField(field + '_MULT').validations = {};

        self.masterDetailEdit();
    };

    self.widgetCancel = function(widget, field) {
        widget.getField(field).showOnForm = true;
        widget.getField(field + '_MULT').showOnForm = false;
        widget.getField(field + '_MULT').validations = {
            "required": {}
        };

        self.masterDetailCancelForm();
    };

    self.defaultEditAuditRow = function(widget, operator) {
        var nrorg = operator[0].NRORG;
        var cdoperador = operator[0].CDOPERADOR;
        var dtinclusao = moment(new Date()).format('DD/MM/YYYY HH:mm');

        if (widget.currentRow.__is_new) {
            widget.currentRow.NRORG = nrorg;
            widget.currentRow.NRORGINCLUSAO = nrorg;
            widget.currentRow.CDOPERINCLUSAO = cdoperador;
            widget.currentRow.DTINCLUSAO = dtinclusao;
        } else {
            widget.currentRow.NRORGULTATU = nrorg;
            widget.currentRow.CDOPERULTATU = cdoperador;
            widget.currentRow.DTULTATU = dtinclusao;
        }
    };

    self.checkRequirements = function(widget) {
        if (!widget.isValid()) {
            ScreenService.showMessage('Preencha todos os campos marcados com asterisco!');
            return false;
        }

        return true;
    };

    self.validaInscricao = function(widget) {
        if (isNaN(widget.currentRow.NRINSCRICAOPARC)) {
            ScreenService.showMessage("Número de Inscrição Juridica deve ser um número válido.");
            return false;
        }

        return true;
    };

    self.checaInscricao = function(widget) {
        var nrinscricaoparc = widget.currentRow.NRINSCRICAOPARC;
        var checagem = true;

        if ((nrinscricaoparc !== null) && (nrinscricaoparc !== undefined)) {
            checagem = self.validaInscricao(widget);
            if (checagem) {
                if (nrinscricaoparc.length == 11) {
                    widget.currentRow.CDTIPOINSCRICAO = "CPF";
                } else if (nrinscricaoparc.length == 14) {
                    widget.currentRow.CDTIPOINSCRICAO = "CNPJ";
                } else {
                    widget.currentRow.CDTIPOINSCRICAO = "LIVRE";
                }
            }
        } else {
            widget.currentRow.CDTIPOINSCRICAO = "LIVRE";
        }

        return checagem;
    };

    self.saveOperador = function(widget) {
        if (widget.isValid() && self.checaInscricao(widget)) {
            CRM_Operador_Repository.retornaOperadorLogado().then(function(operator) {
                var senha = widget.currentRow.CDSENHAOPERWEB;
                var confirmarSenha = widget.currentRow.CONFIRMAR_SENHA;

                if (senha === confirmarSenha) {

                    self.defaultEditAuditRow(widget, operator);
                    widget.currentRow.IDOPERATIVO = (widget.currentRow.IDATIVO == 'S') ? 'A' : 'I';

                    if (widget.currentRow.__is_new === false) {
                        widget.currentRow.OLDNRINSCRICAOPARC = widget.backupRow.NRINSCRICAOPARC;
                        widget.currentRow.OLDCDCLIENTE = widget.backupRow.CDCLIENTE;
                        widget.currentRow.OLDCDSENHAOPERWEB = widget.backupRow.CDSENHAOPERWEB;
                        widget.currentRow.OLDNMOPERADOR = widget.backupRow.NMOPERADOR;
                    }

                    var cdOper = widget.currentRow.CDOPERADOR;

                    if (!widget.currentRow.NRINSCRICAOPARC) {
                        widget.currentRow.NRINSCRICAOPARC = cdOper;
                    } else {
                        widget.currentRow.NRINSCRICAOPARC = widget.currentRow.NRINSCRICAOPARC;
                    }
                    this.retornaFiliaisOperadorLogado().then(function(response) {
                        if (getLocalVar('IDOPERMASTER') === "N") {
                            widget.currentRow.FILIAIS = response.map(function(filial) {
                                return filial.CDFILIAL;
                            });
                            widget.currentRow.IDOPERMASTERLOGADO = "N";
                        }
                        widget.saveRow().then(function(response) {
                            widget.currentRow.changed = false;
                            widget.currentRow.__is_new = false;

                            if (response.dataset) {
                                if (response.dataset.DADOS) {
                                    if (response.dataset.DADOS.NRPARCNEGOCIO) {
                                        widget.currentRow.NRPARCNEGOCIO = response.dataset.DADOS.NRPARCNEGOCIO;
                                        widget.currentRow.CDOPERADOR = response.dataset.DADOS.CDOPERADOR;
                                    }
                                    //widget.currentRow.CDSENHOPER = response.dataset.DADOS.CDSENHOPER;
                                    if (response.dataset.DADOS.CDSENHAOPERWEB) {
                                        widget.currentRow.CDSENHAOPERWEB = response.dataset.DADOS.CDSENHAOPERWEB;
                                        widget.currentRow.CONFIRMAR_SENHA = response.dataset.DADOS.CDSENHAOPERWEB;
                                    }
                                }
                            }
                            self.blockCdoperadorField(widget, true);

                            self.changeWidgetVisibility(widget.widgets, true);
                            widget.list();
                            ScreenService.closeSwipe();
                            widget.reload();
                            this.setVisibilityFilialTab(widget);
                        }.bind(this)).catch(function(error) {
                            if (error.data && error.data.error) {
                                //tratando restrição unique de email
                                if (error.data.error.indexOf('USR_ORG_SAAS.UN_OPERADOR') !== -1)
                                    ScreenService.showMessage("Operação bloqueada! <br><br> Já existe um operador com este email.");
                                else
                                    ScreenService.showMessage(error.data.error);
                            }
                        });
                    }.bind(this));
                } else {
                    ScreenService.showMessage("As senhas não correspondem.");
                }
            }.bind(this));
        }
    };

    self.savePerfil = function(widget) {
        if (self.checkRequirements(widget)) {
            CRM_Operador_Repository.retornaOperadorLogado().then(function(operator) {
                self.defaultEditAuditRow(widget, operator);

                if (widget.getField("NMPERFILACESSO_MULT").selectWidget) {

                    var rows = widget.getField("NMPERFILACESSO_MULT").selectWidget.getCheckedRows();

                    rows.forEach(function(row) {
                        row.NRPARCNEGOCIO = widget.currentRow.NRPARCNEGOCIO;
                        row.NRPERFACESSOPAI = row.NRPERFILACESSO;
                        row.IDATIVO = widget.currentRow.IDATIVO;
                        row.DTVALPERFILPARC = "";
                        row.__new_code_pk = "PERFACESSPARC";
                        row.__table_pk = "NRPERFACESSPARC";
                        row.__cast_number = true;
                        row.__new_code_length = 1;
                        row.__new_code_column_size = 20;
                        row.__is_new = true;
                        row.changed = true;
                        row.__using_unit = false;
                    });

                    InflightorGeneral.callRestEngine("/crm_perfacessparcCompleto/save", "DataSet", rows).then(function(success) {
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload();
                    }, function(error) {
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload();
                    });
                } else {
                    widget.saveRow().then(function() {
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload();
                    }, function(error) {
                        var msg = (error.data && error.data.error) ?
                            msg = error.data.error : 'Registro já existente!';
                        ScreenService.showMessage(msg);
                    });
                }
            });
        }
    };

    self.saveFilial = function(widget) {
        if (self.checkRequirements(widget)) {
            CRM_Operador_Repository.retornaOperadorLogado().then(function(operator) {
                self.defaultEditAuditRow(widget, operator);

                if (widget.getField("NMFILIAL_MULT").selectWidget) {

                    var rows = widget.getField("NMFILIAL_MULT").selectWidget.getCheckedRows();

                    rows.forEach(function(row) {
                        row.IDATIVO = widget.currentRow.IDATIVO;
                        row.__new_code_pk = "FILIAL";
                        row.__table_pk = "CDFILIAL";
                        row.__cast_number = true;
                        row.__new_code_length = 1;
                        row.__new_code_column_size = 12;
                        row.__is_new = true;
                        row.changed = true;
                        row.__using_unit = false;
                    });
                    widget.saveRow().then(function() {
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload();
                    }, function(error) {
                        var msg = (error.data && error.data.error) ?
                            msg = error.data.error : 'Registro já existente!';
                        ScreenService.showMessage(msg);
                    });
                } else {
                    widget.saveRow().then(function() {
                        ScreenService.closeSwipe();
                        widget.activate();
                        widget.reload();
                    }, function(error) {
                        var msg = (error.data && error.data.error) ?
                            msg = error.data.error : 'Registro já existente!';
                        ScreenService.showMessage(msg);
                    });
                }
            });
        }
    };

    /*self.iniciarDataSourceChild = function(widget) {
        var widPerf = widget.widgets[0];
        var widFili = widget.widgets[1];

        self.initTabGeneric(widPerf, "NRPARCNEGOCIO");
        self.initTabGeneric(widFili, "CDOPERADOR");
    };*/

    self.initTabGeneric = function(widget, field) {
        if (typeof widget === "string") {
            widget = templateManager.container.getWidget(widget);
        }
        if (widget.getParent().currentRow[field]) {
            widget.dataSourceFilter[0].value = widget.getParent().currentRow[field];
            widget.reload();
        }
    };

    self.changeWidgetVisibility = function(widgets, isVisible) {
        widgets.forEach(function(widget) {
            if (widget.name !== "wdgSenha") {
                if (widget.name === 'FILIOPER' || widget.name === "PERFACESSO") {
                    widget.isVisible = isVisible;
                    widget.enabled = true;
                    widget.reload();
                }
            }
        });
    };

    self.formChangeTemplate = function() {
        var widget = templateManager.container.getActiveWidget();

        if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

        if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
        if (!widget.templateToView) widget.templateToForm = "widget/form-view.html";

        if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
        else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

        templateManager.updateTemplate();

        return true;
    };

    self.goBackToDashboard = function() {
        ScreenService.openWindow('dashboard');
    };

    self.masterDetailCancelForm = function() {
        var widget = templateManager.container.getActiveWidget();

        self.changeWidgetVisibility(widget.widgets, true);

        if (widget.currentRow.__is_new) {
            var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
            if (indexOfCurrentRow !== -1) {
                widget.dataSource.data.pop();
            }
            if (!self.formChangeTemplate()) widget.list();
            else widget.list();

            ScreenService.closeSwipe();
        } else {
            Util.extend(widget.currentRow, widget.backupRow);
            self.formChangeTemplate();
            widget.view();
        }
    };

    self.masterDetailClosePopup = function() {
        var popup = templateManager.container.getActiveWidget();

        ScreenService.closePopup();
        popup.deactivate();
        popup.parentWidget.currentWidget = popup.parentWidget;
        popup.parentWidget.activate();
    };

    self.masterDetailDelete = function() {
        var widget = templateManager.container.getActiveWidget();
        ScreenService.confirmMessage("Deseja deletar o registro?", "CONFIRMATION", function() {
            widget.remove();
            self.masterDetailGoBackToList();
        }.bind(this));
    };

    self.masterDetailDeleteOper = function(row) {

        var widget = templateManager.container.getActiveWidget();

        ScreenService.confirmMessage("Deseja deletar o registro?", "CONFIRMATION", function() {
            InflightorGeneral.callRestEngine("/operadorTelaPrincipal/remove", "Row", row).then(function(success) {
                ScreenService.closeSwipe();
                widget.activate();
                widget.reload();
            }, function(error) {
                var msg = (error.data && error.data.error) ?
                    msg = error.data.error : 'Registro já existente!';
                ScreenService.showMessage(msg);
            });
        }.bind(this));
    };

    self.masterDetailEdit = function() {
        var widget = templateManager.container.getActiveWidget();

        self.changeWidgetVisibility(widget.widgets, false);
        self.formChangeTemplate();

        widget.backupRow = Util.clone(widget.currentRow);
        widget.edit();
    };

    self.masterDetailGoBackToList = function() {
        var widget = templateManager.container.getActiveWidget();

        ScreenService.closeSwipe();
        widget.list();
    };

    self.masterDetailGoBackToParent = function() {
        var widget = templateManager.container.getActiveWidget();

        ScreenService.closeSwipe().then(function() {
            widget.getParent().list();
            widget.getParent().activate();
        });
    };

    self.masterDetailNewRow = function() {
        var widget = templateManager.container.getActiveWidget();

        self.changeWidgetVisibility(widget.widgets, false);

        widget.newRow();
    };

    /*self.masterDetailOpenPopup = function(name) {
        var widget = templateManager.container.getActiveWidget();

        var popup = widget.container.getWidget(name);
        popup.parentWidget = widget;
        ScreenService.openPopup(popup);
        ZhGridFilter.startFiltering();
    };*/

    self.masterDetailSave = function() {
        var widget = templateManager.container.getActiveWidget();

        self.changeWidgetVisibility(widget.widgets, true);
        self.formChangeTemplate();

        if (widget.isValid()) {
            widget.currentRow.changed = true;

            if (widget.currentRow.__is_new) {
                widget.currentRow.NRORGINCLUSAO = (widget.currentRow.NRORGINCLUSAO) ? widget.currentRow.NRORGINCLUSAO : getLocalVar('NRORG');
                widget.currentRow.CDOPERINCLUSAO = (widget.currentRow.CDOPERINCLUSAO) ? widget.currentRow.CDOPERINCLUSAO : getLocalVar('CDOPERADOR');
            } else {
                widget.currentRow.NRORGULTATU = (widget.currentRow.NRORGULTATU) ? widget.currentRow.NRORGULTATU : getLocalVar('NRORG');
                widget.currentRow.CDOPERULTATU = (widget.currentRow.CDOPERULTATU) ? widget.currentRow.CDOPERULTATU : getLocalVar('CDOPERADOR');
            }

            return widget.saveRow().then(function(response) {
                widget.currentRow.changed = false;
                widget.currentRow.__is_new = false;
                ScreenService.showMessage("Registro salvo com sucesso");
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
            });
        }
    };

    self.unMask = function(columnName) {
        var widget = templateManager.container.getActiveWidget();

        if (widget.currentRow[columnName] && typeof widget.currentRow[columnName] === 'string' && widget.currentRow[columnName].indexOf('R$') != -1)
            widget.currentRow[columnName] = $.currency.unmaskString(widget.currentRow[columnName], {
                "thousands": "."
            });
        if (widget.currentRow[columnName] && typeof widget.currentRow[columnName] === 'string' && widget.currentRow[columnName].indexOf('%') != -1)
            widget.currentRow[columnName] = parseFloat(widget.currentRow[columnName].replace(',', '.'));
    };

    self.wizardChangeFooter = function() {
        var widget = templateManager.container.getActiveWidget();

        widget.container.footer = (widget.container.footer == "component/footer.html") ?
            "component/footer-wizard.html" : "component/footer.html";
    };

    self.wizardGoBack = function() {
        wizardEngine.previous();
    };

    self.wizardGoNext = function() {
        wizardEngine.next();
    };

    self.getValorEmReais = function(value) {
        var pad = "00";
        var valorFatura = value.split('.');

        if (valorFatura[1] === undefined) {
            valorFatura[1] = "";
        }
        valorFatura[1] = String(valorFatura[1]) + pad.substring(0, pad.length - String(valorFatura[1]).length);

        if (valorFatura[0] === '') {
            valorFatura[0] = "0";
        }
        var aux = "";
        for (var i = valorFatura[0].length - 1; i >= 0; i--) {
            if (((valorFatura[0].length - 1 - i) % 3 === 0) && (valorFatura[0].length - 1 !== i)) {
                aux += ".";
            }
            aux += valorFatura[0][i];
        }

        aux = aux.split("").reverse().join("");
        var valorEmReais = "R$ " + aux + "," + valorFatura[1];
        return valorEmReais;
    };

    /*self.validateFilterPopup = function(args)
    {
        var filterPopup = templateManager.currentWidget;
        unidade = filterPopup.currentRow.CDFILIAL;
        if(!filterPopup.isValid())
            args.e.preventDefault();
    };*/

    this.wdgSenhaSetCurrentRow = function(widget) {
        widget.currentRow = widget.getParent().dataSource.data[0];
    };

    this.wdgSenhaEditar = function(widget) {
        InflightorGeneral.widgetEdit(widget);
        widget.currentRow.CDSENHAOPERWEB = "";
        widget.currentRow.CONFIRMAR_SENHA = "";
    };

    this.wdgSenhaSalvar = function(widget) {
        if (widget.isValid()) {
            var senha = widget.currentRow.CDSENHAOPERWEB;
            var confirmarSenha = widget.currentRow.CONFIRMAR_SENHA;

            if (senha === confirmarSenha) {
                restEngine.post('/operador/saveSenha', {
                        'requestType': 'Row',
                        'row': widget.currentRow
                    })
                    .then(function(response) {
                        if (response.dataset) {
                            if (response.dataset.DADOS) {
                                if (response.dataset.DADOS.CDSENHAOPERWEB) {
                                    widget.currentRow.CDSENHAOPERWEB = response.dataset.DADOS.CDSENHAOPERWEB;
                                    widget.currentRow.CONFIRMAR_SENHA = response.dataset.DADOS.CDSENHAOPERWEB;
                                }
                            }
                            widget.getParent().enabled = true;
                            widget.getParent().reload();
                            self.changeWidgetVisibility(widget.getParent().widgets, true);
                            self.wdgSenhaSetCurrentRow(widget);
                            widget.template = "widget/form-view.html";
                            widget.view();

                        }
                    });
            } else {
                ScreenService.showMessage("As senhas não correspondem.");
            }
        }
    };

    this.setOrganOper = function(args) {
        var widget = args;
        if (widget.isValid()) {
            widget.currentRow.NRORG = getLocalVar('CRM_NRORG') ? getLocalVar('CRM_NRORG') : " ";
            widget.currentRow.CDOPERADOR = widget.getParent().currentRow.CDOPERADOR;
            widget.currentRow.CDOPERINCLUSAO = getLocalVar('LOGIN_CDOPERADOR');
            return InflightorGeneral.saveCrud(widget).then(function(response) {
                widget.reload();
            });

        }
    };

    this.setVisibilityFilialTab = function(widget) {
        /*var visible;
        if (getLocalVar('IDOPERMASTER') === "S") {
            visible = true;
        } else {
            visible = false;
        }*/

        var visible = true;
        if (widget.currentRow.__is_new !== true) {
            widget.widgets.forEach(function(tab) {
                if (tab.name === "FILIOPER") {
                    tab.isVisible = visible;
                }
            });
        }
        this.setReadOnlyOperMasterField(widget, !visible);
    };

    this.setReadOnlyOperMasterField = function(widget, readOnly) {
        widget.fields.forEach(function(field) {
            if (field.name === "IDOPERMASTER") {
                field.readOnly = readOnly;
            }
        });
    };

    /**
     * Seta os filtros padrões
     * 
     * @param {object} widget widget de OPERADOR
     */
    this.defaultFilters = function(widget) {
        var filter = widget.__filterPopup;
        var NMFILIAL, P_CDFILIAL, filiaisFilter, filialField = widget.getField('P_CDFILIAL');
        var idopermaster = getLocalVar('IDOPERMASTER');

        if (idopermaster === "N") {
            this.retornaFiliaisOperadorLogado().then(function(response) {
                P_CDFILIAL = response.map(function(filial) {
                    return filial.CDFILIAL;
                });
                NMFILIAL = response.map(function(filial) {
                    return filial.NMFILIAL;
                });
                filter.currentRow.P_CDFILIAL = P_CDFILIAL;
                filter.currentRow.NMFILIAL = NMFILIAL;

                filter.getField('P_CDFILIAL').reload();

                filiaisFilter = {
                    "name": "CDFILIAL",
                    "operator": "IN",
                    "value": angular.copy(P_CDFILIAL)
                };
                filialField.dataSourceFilter.push(filiaisFilter);

            });
        } else {
            this.retornaFiliaisOrg().then(function(response) {
                P_CDFILIAL = response.map(function(filial) {
                    return filial.CDFILIAL;
                });
                NMFILIAL = response.map(function(filial) {
                    return filial.NMFILIAL;
                });
                filter.currentRow.P_CDFILIAL = P_CDFILIAL;
                filter.currentRow.NMFILIAL = NMFILIAL;

                filter.getField('P_CDFILIAL').reload();
            });
        }

    };

    this.retornaFiliaisOrg = function() {
        var filter = [
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORG')
            }
        ];
        return localStorageEngine.download("product#/crud/filial", filter);
    };

    this.retornaFiliaisOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": getLocalVar('CDOPERADOR')
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": getLocalVar('NRORG')
            }
        ];
        return localStorageEngine.download("product#/crm_filioperCompleto", filter);
    };

    this.callDefaultFilters = function(widget) {
        var field = widget.getField('P_CDFILIAL');

        if (Util.isEmptyOrBlank(field.value())) {
            this.defaultFilters(templateManager.container.getWidget('OPERADOR'));
        }
    };

    /*Replica mudanças no IDATIVO para o IDOPERATIVO*/
    this.changeIdoperativo = function(widget) {
        widget.currentRow.IDOPERATIVO = (widget.currentRow.IDATIVO === 'S') ? 'A' : (widget.currentRow.IDATIVO === 'N') ? 'I' : '';
    };

    this.prepareAtivoFilter = function(widget) {
        widget.dataSourceFilter.forEach(function(filter, index, filters) {
            if (filter.name === "IDATIVO") {
                filters.push({
                    "name": "IDOPERATIVO",
                    "operator": "=",
                    "static": true,
                    "value": (filter.value === 'S') ? 'A' : 'I'
                });
            }
        });
    };
    /*Fim*/
}

Configuration(function(ContextRegister) {
    ContextRegister.register('CRM_Operador_Controller', CRM_Operador_Controller);
});

// FILE: ./assets/js/controllers/dashboardController.js
var InflightorDashboardController = function(templateManager, restEngine, ZhUtil, ZHPromise, InflightorGeneral) {
    var self = this;

    this.onEnterHome = function() {
        InflightorGeneral.validateInflightorSession().then(function() {
            var promises = [];

            promises.push(situacaoOperworkflow());
            promises.push(vooEstimado());
            promises.push(vooAtendido());

            promises.push(operworkflow());
            promises.push(situacao());
            promises.push(pendencia());

            ZHPromise.all(promises).then(function(response) {
                var situacaoOperworkflowDash = response[0].dataset.situacao_operworkflow_dash[0];
                var vooEstimadoDash = response[1].dataset.voo_estimado_dash[0];
                var vooAtendidoDash = response[2].dataset.voo_atendido_dash[0];

                var operworkflowData = response[3].dataset.operworkflow_workflow_dash;
                var situacaoData = response[4].dataset.situacao_chart_operworkflow_dash;
                var pendenciaData = response[5].dataset.pendencia_worflow_chart_dash;

                if (!Util.isUndefined(situacaoOperworkflowDash)) {
                    var widgetProcessosPendentes = templateManager.container.getWidget('processos_pendentes');
                    var widgetProcessosAprovados = templateManager.container.getWidget('processos_aprovados');
                    var widgetProcessosReprovados = templateManager.container.getWidget('processos_reprovados');
                    var widgetProcessosExpirados = templateManager.container.getWidget('processos_expirados');
                    var widgetVoosAtendidos = templateManager.container.getWidget('voos_atendidos');
                    var widgetVoosEstimados = templateManager.container.getWidget('voos_estimados');

                    var widgetOperworkflowDash = templateManager.container.getWidget('pendencias_workflow');
                    var widgetSituacaoDash = templateManager.container.getWidget('chart_status_processos');
                    var widgetPendencia = templateManager.container.getWidget('chart_pendencias_workflow');

                    widgetProcessosPendentes.highlight = situacaoOperworkflowDash.PENDENTES;
                    widgetProcessosAprovados.highlight = situacaoOperworkflowDash.APROVADOS;
                    widgetProcessosReprovados.highlight = situacaoOperworkflowDash.REPROVADOS;
                    widgetProcessosExpirados.highlight = situacaoOperworkflowDash.EXPIRADOS;
                    widgetVoosAtendidos.highlight = vooAtendidoDash.QTDVOOATEND;
                    widgetVoosEstimados.highlight = vooEstimadoDash.QTDVOOESTIMADO;

                    widgetSituacaoDash.dataSource.data = operworkflowData;
                    widgetSituacaoDash.dataSource.data = situacaoData;
                    widgetPendencia.dataSource.data = pendenciaData;
                }
            });
        });
    };

    var situacaoOperworkflow = function() {
        return restEngine.post("product#/crud/situacao_operworkflow_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }]
        });
    };

    var vooEstimado = function() {
        return restEngine.post("product#/crud/voo_estimado_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }]
        });
    };

    var vooAtendido = function() {
        return restEngine.post("product#/crud/voo_atendido_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }]
        });
    };

    var operworkflow = function() {
        return restEngine.post("product#/crud/operworkflow_workflow_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
                }]
        });
    };

    var situacao = function() {
        return restEngine.post("product#/crud/situacao_chart_operworkflow_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
                }]
        });
    };

    var pendencia = function() {
        return restEngine.post("product#/crud/pendencia_worflow_chart_dash", {
            "requestType": "FilterData",
            "filter": [{
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
                }]
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('InflightorDashboardController', InflightorDashboardController);
});

// FILE: ./assets/js/controllers/ger00003_unidadeController.js
var UnidadeController = function(ScreenService, UnidadeService, Parametros, eventAggregator) {

    this.onEnter = function(widget) {
        widget.reload();
    };

    this.afterMoveRow = function(unit) {
        var message = "Deseja confirmar a troca de unidade?";
        ScreenService.confirmMessage(message, "question", function() {
            changeUnit(unit);
        });
    };

    var changeUnit = function(unit) {
        UnidadeService.alterarEstrutura(unit.CDFILIAL, unit.NRORG).then(function() {
            UnidadeService.updateMongoSession(unit.NMFILIAL, unit.CDFILIAL).then(function() {
                updateLocalSession(unit);
            }, callbackError);
        }, callbackError);
    };

    var updateLocalSession = function(unit) {
        Parametros.retornaDataProcessamento(unit.CDFILIAL, unit.NRORG).then(function(dtprocessa) {
            unit.DTPROCESSA = dtprocessa;
            setLocalVars(unit);
            buildUserAndHeader();
            eventAggregator.publish("onUnitChange", unit);
            ScreenService.openWindow('product#dashboard');
        }, callbackError);
    };

    function buildUserAndHeader() {
        var nmoperador = getLocalVar('NMOPERADOR');
        var dtprocessa = getLocalVar('DTPROCESSA');
        var cdfilial = getLocalVar('CDFILIAL');
        var nmfilial = getLocalVar('NMFILIAL');
        var unidade = cdfilial + " - " + nmfilial;
        var headerInfo = dtprocessa + " | " + unidade;
        ScreenService.buildUserData(nmoperador, [dtprocessa, unidade], userMethod);
        ScreenService.setHeaderInfo(headerInfo);
    }

    function userMethod() {
        // @todo create user profile window
    }

    var setLocalVars = function(unidade) {
        setLocalVar("CDFILIAL", unidade.CDFILIAL);
        setLocalVar("NMFILIAL", unidade.NMFILIAL);
        setLocalVar("DTPROCESSA", unidade.DTPROCESSA);
    };

    var callbackError = function(err) {
        ScreenService.showMessage(JSON.stringify(err.data));
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UnidadeController', UnidadeController);
});

// FILE: ./assets/templates/inf-select-multiple/dist/inf-select-multiple.js
window.templateUrls.push({
    "package": "inf-select-multiple",
    "baseUrl": "bower_components/inf-select-multiple/dist/templates/"
});

function InfSelectMultipleController($scope, DecodeValueFieldService) {
    var field = $scope.field;

    var defaultBuildParams = {
        fields: field.fields,
        fieldsSeparator: '-',
        valuesSeparator: '_'
    };
    var buildParams = field.concatenatedValueString ? field.concatenatedValueString : defaultBuildParams;
    buildParams.fields = Util.isArray(buildParams.fields) ? buildParams.fields : defaultBuildParams.fields;
    buildParams.fieldsSeparator = !Util.isArray(buildParams.fieldsSeparator) ? buildParams.fieldsSeparator : defaultBuildParams.fieldsSeparator;
    buildParams.valuesSeparator = !Util.isArray(buildParams.valuesSeparator) ? buildParams.valuesSeparator : defaultBuildParams.valuesSeparator;

    field.valueStrBuildParams = buildParams;

    field.isSelectReadOnly = function() {
        return field.checked || field.isReadOnly();
    };

    field.createFilter = function() {
        if (buildParams && buildParams.fields && buildParams.fields.length) {
            var valueStr = field.value();

            if (valueStr != field.valueToAll) {
                var valuesSeparator = buildParams.valuesSeparator;
                var valueArray = typeof(valueStr) == 'string' ? valueStr.split(valuesSeparator) : [];
                var filterName = field.name;

                var filter = {
                    "name": filterName,
                    "operator": 'IN',
                    "value": valueArray
                };
                return [filter];
            } else {
                return [];
            }
        } else {
            return [];
        }
    };

    var decodeInfSelectMultiple = function(field, fieldRow) {
        if (fieldRow === field.valueToAll) {
            return field.checkboxLabel;
        } else {
            let valueParams = field.concatenatedValueString;
            if (valueParams && valueParams.fields && valueParams.valuesSeparator) {
                let valueFields = valueParams.fields;
                let valuesSeparator = valueParams.valuesSeparator;
                let fieldsSeparator = valueParams.fieldsSeparator;

                let getValueFromRow = function(row) {
                    if (valueFields.length == 1) {
                        return row[valueParams.fields[0]];
                    }
                    if (valueFields.length > 1 && fieldsSeparator) {
                        return valueFields.map(function(valueField) {
                            return row[valueField];
                        }).join(fieldsSeparator);
                    }
                    return null;
                };
                let isValueSelected = function(value) {
                    let selectedValues = fieldRow.split(valuesSeparator);
                    return (selectedValues.indexOf(value) != -1);
                };
                let isRowSelected = function(row) {
                    let rowValue = getValueFromRow(row);
                    return ((rowValue !== null) && isValueSelected(rowValue));
                };
                let getRowDescription = function(row) {
                    return row[field.descriptionField];
                };

                let selectedRows = field.dataSource.data.filter(isRowSelected);
                let selectedRowsDescriptions = selectedRows.map(getRowDescription);

                return selectedRowsDescriptions.join(", ");
            }
        }
        return null;
    };
    DecodeValueFieldService.register('inf-select-multiple.html', decodeInfSelectMultiple);

}

//SELECT
function InfSelectMultipleSelectController($scope, $rootScope, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, metaDataFactory, EventChildPropagation) {

    var buildMultipleSelectValueStr = function(field, rows, buildParams) {
        var valuesArray = [];
        rows.forEach(function(item) {
            var itemArray = [];
            buildParams.fields.forEach(function(field) {
                itemArray.push(item[field]);
            });
            var itemString = itemArray.join(buildParams.fieldsSeparator);
            valuesArray.push(itemString);
        });
        field.valueAsArray = valuesArray;
        return valuesArray.length ? valuesArray.join(buildParams.valuesSeparator) : null;

    };

    function overrideSelectWidgetUnselectAllFunction(field) {
        var originalFunction = field.selectWidget.unSelectAll;
        field.selectWidget.unSelectAll = function() {
            originalFunction();
        };
    }

    var _openField = function(field) {

        var defer = ZHPromise.defer();
        var actions = [{
            "label": ScreenService.i18n("Cancel"),
            "showAsAction": "back",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "hideIcon": true,
            "events": [{
                "name": "ActionEvent",
                "code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
                "id": 9998
			}]
		}, {
            "label": ScreenService.i18n("Ok"),
            "showAsAction": "never",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "events": [{
                "name": "ActionEvent",
                "code": "args.owner.widget.setSelected(args.owner.widget.dataSource.getCheckedRows(), args.owner.widget.selectField); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
                "id": 9999
			}]
		}];

        if (field.showAddButtonAs == 'action') {
            actions.push({
                "icon": "plus",
                "showAsAction": "always|edit|checked_rows|view",
                "isVisible": true,
                "showOnForm": true,
                "showOnList": true,
                "events": [{
                    "name": "ActionEvent",
                    "code": "args.owner.widget.onAdd()",
                    "id": 9999
				}]
            });
        }

        innerSearch("", field);

        var template;
        if (!field.showSelectAs) {
            template = "widget/grid.html";
        } else if (field.showSelectAs === "tree") {
            template = "widget/tree/grid.html";
        } else {
            template = "widget/" + field.showSelectAs + ".html";
        }

        var fields;
        if (field.fields && field.fields.length) {
            fields = field.fields;
        } else {
            fields = [{
                "label": field.sourceLabel,
                "name": field.descriptionField,
                "showOnList": true
			}];
        }

        var updateCheckedRows = function() {
            if (field && field.multipleSelectCheckedRows) {
                field.dataSource.checkedRows = field.multipleSelectCheckedRows;
            }
            defer.resolve();
        };

        var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
        if (!field.selectWidget) {
            var widgetEvents = [{
                "name": "WidgetAfterMoveRow",
                "code": "args.owner.toggle(args.row, args.owner);",
                "id": 9999
			}];
            field.selectWidget = metaDataFactory.widgetFactory({
                "id": 9999,
                "name": "select_multiple_widget",
                "label": ScreenService.i18n(field.label),
                "template": template,
                "container": field.widget.container,
                "isVisible": true,
                "popupNoBlock": false,
                "showDescriptionOrder": false,
                "floatingControl": {
                    "openSearch": !Util.isMobile(),
                    "defaultSearchField": field.defaultSearchField || field.descriptionField,
                    "pdfAction": false,
                    "xlsAction": false,
                    "csvAction": false,
                    "searchAction": true,
                    "customizationAction": {
                        "filter": false,
                        "layout": false,
                        "view": false
                    }
                },

                "parentProperty": field.parentProperty,
                "primaryProperty": field.primaryProperty,
                "expandingProperty": field.expandingProperty,
                "breadcrumbProperty": field.breadcrumbProperty,
                "selectCascade": field.selectCascade,
                "childrenDependency": field.childrenDependency,

                "order": field.order,
                "reverse": field.reverse,
                "bringGroupsCollapsed": field.bringGroupsCollapsed,
                "selectField": field,
                "showCheckbox": true,
                "onAdd": field.onAdd,
                "showAddButtonAs": field.showAddButtonAs,
                "addButtonText": field.addButtonText,
                "noMatchText": field.noMatchText,
                "searchNoMatchText": field.searchNoMatchText,
                "itemsPerPage": field.itemsPerPage,
                "searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
                "hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
                "onDemand": field.onDemand,
                "searchCriteria": field.searchCriteria,

                "fields": fields,
                "dataSource": field.dataSource,
                "dataSourceFilter": field.dataSourceFilter,
                "events": EventChildPropagation.propagate(widgetEvents, field),
                "actions": actions,

                "styles": field.selectWidgetStyles
            });
            field.selectWidget.setSelected = setSelected;
            field.selectWidget.afterClosePopup = afterClosePopup;
            isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
            overrideSelectWidgetUnselectAllFunction(field);
        }

        if (isToReload) {
            field.selectWidget.searchCriteria = {};
            field.selectWidget.searchList = "";
            field.reload().then(updateCheckedRows, updateCheckedRows);
        } else {
            updateCheckedRows();
        }

        ScreenService.openPopup(field.selectWidget);
        return defer.promise;
    };

    $scope.openField = function(field) {
        var e = false;
        if (!field.isSelectReadOnly()) {
            if (field.beforeSelectOpen) {
                e = field.beforeSelectOpen();
                if (e.isDefaultPrevented()) {
                    return;
                }
            }
            // remove focus from element
            field.element.blur();
            window.getSelection().removeAllRanges(); //bug webkit

            _openField(field).then(function() {
                if (field.afterSelectOpen) {
                    field.afterSelectOpen();
                }
            });
        }
    };

    function innerSearch(selectListSearch, field) {
        if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
            var dataSource = field.dataSource;
            var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
            dataSource.filter(filter, function(data) {
                dataSource.data = data;
                templateManager.updateTemplate();
            });
        }
    }

    var setChanged = function(field) {
        field.widget.currentRow.changed = true;
    };

    function setSelected(rows, field) {
        ScreenService.closePopup();

        field.multipleSelectValueStr = buildMultipleSelectValueStr(field, rows, field.valueStrBuildParams);
        field.multipleSelectCheckedRows = rows;
        changeByValueStr(field);
        setChanged(field);

        if (field._afterSelected) {
            field._afterSelected(row);
        }

        if (field.change) field.change();
    }

    function afterClosePopup(field) {
        if (field.element) {
            field.element.focus();
        }
    }

    function changeByValueStr(field) {
        field.widget.currentRow[field.name] = field.multipleSelectValueStr;
    }

    $scope.maintainWindow = function(field) {
        if (field.maintainWindow) {
            var maintainWindowFilter = SelectService.buildMaintainFilter(field);
            ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
        }
    };

    $scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);

    function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
        if ($scope.field.selectWidget) {
            $scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
        }
    }

    this.hasSelectedRows = function hasSelectedRows(field) {
        var selectedRows = field.value();
        return selectedRows && selectedRows.length > 0;
    };
}
var infSelectMultipleSelectDirective = function($compile, $timeout, templateManager, Query, ZHPromise, ZhUtil) {
    return {
        priority: 1,
        link: function($scope, element) {
            // these options must be replaced by dataSource values
            var options = [];
            var field = $scope.field;

            var buildMultipleSelectValueStr = function(field, rows, buildParams) {
                var valuesArray = [];
                rows.forEach(function(item) {
                    var itemArray = [];
                    buildParams.fields.forEach(function(field) {
                        itemArray.push(item[field]);
                    });
                    var itemString = itemArray.join(buildParams.fieldsSeparator);
                    valuesArray.push(itemString);
                });
                return valuesArray.length ? valuesArray.join(buildParams.valuesSeparator) : null;

            };

            var getValueField = function() {
                return field.valueField ? field.valueField : field.valueFields;
            };

            var getDescriptionField = function() {
                return field.descriptionField;
            };

            element.on('keydown', function(e) {
                return e.ctrlKey || e.keyCode == 9 || (e.keyCode >= 112 && e.keyCode <= 126); // TAB key
            });

            element.on('input', function(e) {
                element.text("");
                return false;
            });

            $scope.blurInput = function() {
                element.blur();
            };

            $scope.removeOption = function(evt) {
                var elm = angular.element(evt.currentTarget);
                if (!field.isReadOnly() && !field.isSelectReadOnly()) {
                    var row = options.splice(elm.index(), 1);
                    field.dataSource.removeCheckedRows(row);
                    var checkedRows = field.dataSource.getCheckedRows();
                    field.multipleSelectValueStr = buildMultipleSelectValueStr(field, checkedRows, field.valueStrBuildParams);
                    field.multipleSelectCheckedRows = checkedRows;
                    $scope.fieldRow = field.multipleSelectValueStr;
                    elm.animate({
                        opacity: 0
                    }, 300, function() {
                        elm.animate({
                            width: 0
                        }, 300, buildOptions);
                    });
                    field.widget.currentRow.changed = true;
                    if (field.change) {
                        field.change();
                    }
                }
                evt.stopPropagation();
            };

            function buildOptions() {
                var container = element.find('.zh-options-container');
                var elementWidth = element.width();
                var optionsWidth = 0;
                var dField = getDescriptionField();

                // clear options added to element
                container.html('');

                // add options to element until options width be greater than element width

                options.forEach(function(item, index) {
                    var option;
                    if (field.isReadOnly()) {
                        option = $compile('<span class="zh-select-item" contenteditable="false"><span>&times;</span>' + item[dField] + '</span>')($scope);
                    } else {
                        option = $compile('<span class="zh-select-item" contenteditable="false" zh-touchstart="removeOption(evt)"><span>&times;</span>' + item[dField] + '</span>')(
                            $scope);
                    }
                    container.append(option);
                    optionsWidth += option[0].clientWidth + 5;
                    if (optionsWidth > elementWidth) {
                        return false;
                    }
                });

                // adjust the options and add the "more" element
                if (optionsWidth > elementWidth) {
                    var more = $compile('<span class="zh-select-item more-items">' + (options.length - container.children().length) + ' mais...</span>')($scope);
                    container.append(more);
                    optionsWidth += more[0].clientWidth + 5;
                    while (optionsWidth > elementWidth && container.children().length > 1) {
                        var lastOption = container.children().eq(-2);
                        optionsWidth -= lastOption[0].clientWidth + 5;
                        lastOption.remove();
                        more.html((options.length - container.children().length + 1) + ' mais...');
                    }
                }
            }

            // bind the window.resize event to rebuild the options if element width changes
            $(window).on("resize.select-multiple", buildOptions);

            function buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows) {
                var vField = getValueField();

                var idsByKey = {};
                if (Util.isArray(vField)) {
                    vField.forEach(function(key) {
                        idsByKey[key] = [];
                    });
                    nonMatchedRows.forEach(function(row) {
                        vField.forEach(function(key) {
                            idsByKey[key].push(row[key]);
                        });
                    });
                } else {
                    idsByKey[vField] = nonMatchedRows;
                }

                var filters = ZhUtil.buildFiltersInField(field);
                var query = Query.build();

                Object.keys(idsByKey).forEach(function(key) {
                    query.where(key).in(idsByKey[key]);
                });

                return filters.concat(query.where());
            }

            function alreadyHasTheRowInCache(row) {
                return field.dataSource.getCheckedRows().some(function(cachedRow) {
                    return Util.parseToArray(getValueField()).every(function(key) {
                        return cachedRow[key] == row[key];
                    });
                });
            }

            function alreadyHasTheRowInTheOptions(row) {
                return options.some(function(option) {
                    return Util.parseToArray(getValueField()).every(function(key) {
                        return option[key] == row[key];
                    });
                });
            }

            function addOption(row) {
                if (!alreadyHasTheRowInTheOptions(row)) {
                    options.push(row);
                    field.dataSource.addCheckedRows(row);
                }
            }

            function loadUnmatchedRowsInDatasource(nonMatchedRows) {

                var filters = buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows);

                return field.dataSource.filter(filters).then(function(data) {
                    if (Array.isArray(data) && data.length) {
                        data.forEach(addOption);
                    }
                });
            }

            // built options property checking the values (selectedRows) against the datasource
            function buildOptionsFromDataSource(selectedRows) {
                var defer = ZHPromise.defer();

                var vField = getValueField();
                var previousCheckedRows = field.dataSource.getCheckedRows();
                field.dataSource.clearCheckedRows();
                options = [];

                if (field.dataSource.data && field.dataSource.data.length && field.multipleSelectValueStr) {
                    var buildParams = field.valueStrBuildParams;
                    var valueFields = buildParams.fields;
                    var selectDataSource = field.selectWidget.dataSource.data;

                    var valuesArray = field.multipleSelectValueStr.split(buildParams.valuesSeparator);
                    var selectedRowsArr = [];

                    valuesArray.forEach(function(valueString, valueIndex) {
                        selectedRowsArr.push({});
                        var valueArray = valueString.split(buildParams.fieldsSeparator);
                        valueFields.forEach(function(valueField, fieldIndex) {
                            selectedRowsArr[valueIndex][valueField] = valueArray[fieldIndex];
                        });
                    });

                    var filterDataSource = function(dataSourceRow) {
                        for (var i in selectedRowsArr) {
                            var selectedRow = selectedRowsArr[i];
                            var isSelected = true;
                            for (let j = 0; j < valueFields.length; j++) {
                                let dataSourceRowValue = String(dataSourceRow[valueFields[j]]);
                                let selectedRowValue = String(selectedRow[valueFields[j]]);
                                if (dataSourceRowValue !== selectedRowValue) {
                                    isSelected = false;
                                    break;
                                }
                            }
                            if (isSelected) {
                                return true;
                            }
                        }
                        return false;
                    };

                    options = selectDataSource.filter(filterDataSource);
                    field.dataSource.checkedRows = options;
                    defer.resolve();
                } else {
                    options = previousCheckedRows;
                    defer.resolve();
                }
                return defer.promise;
            }

            function findRowFromDataSourceOrCheckedRows(dataSourceData, checkedRows, valueField, valueItem) {
                return findInItems(valueField, valueItem, dataSourceData) || findInItems(valueField, valueItem, checkedRows);
            }

            function findInItems(valueField, valueItem, items) {
                return items.filter(function(item) {
                    if (Util.isArray(valueField)) {
                        return valueField.every(function(key) {
                            return item[key] == valueItem[key];
                        });
                    }
                    return item[valueField] == valueItem;
                }).shift();
            }

            var dataSourceWatch = $scope.$watch(function() {
                return field.dataSource.data;
            }, function() {
                if (field.dataSource.data && field.dataSource.data.length) {
                    dataSourceWatch();
                    buildOptionsFromDataSource((field.value() || [])).then(function() {
                        buildOptions();
                    });
                }
            });

            $scope.$watch(function() {
                return field.value();
            }, function(value) {
                buildOptionsFromDataSource(value).then(function() {
                    buildOptions();
                });
            });
        }
    };
};

//CHECKBOX
var infSelectMultipleCheckboxDirective = function(templateManager) {
    return {
        priority: 1,
        link: function($scope, $element) {
            var field = $scope.field;
            var input = $element.find("input");
            var label = $element.find("label");

            field.valueToAll = Util.isUndefined(field.valueToAll) ? "T" : field.valueToAll;

            var getMultipleSelectValue = function() {
                return field.multipleSelectValueStr ? field.multipleSelectValueStr : null;
            };
            var clearMultipleSelectValue = function() {
                field.multipleSelectValueStr = null;
                field.multipleSelectCheckedRows = [];
                if (field.dataSource && field.dataSource.clearCheckedRows) {
                    field.dataSource.clearCheckedRows();
                }
            };

            function toggleValue() {
                if (!input.is(':disabled')) {
                    field.checked = !field.checked;

                    if (!field.readOnly) {
                        $scope.widget.currentRow[field.name] = field.checked ? field.valueToAll : getMultipleSelectValue();
                        $scope.widget.currentRow.changed = true;

                        if (field.change) {
                            field.change();
                        }

                        templateManager.onUpdate();
                    }

                }

                handleCheckedState();
            }

            var handleCheckedState = function() {
                var currentValue = $scope.widget.currentRow[field.name];
                if (currentValue === field.valueToAll) {
                    field.checked = true;
                    clearMultipleSelectValue();
                } else if (field.checked) {
                    currentValue = field.valueToAll;
                    clearMultipleSelectValue();
                } else if (currentValue === getMultipleSelectValue()) {
                    field.checked = false;
                } else {
                    field.checked = false;
                    currentValue = getMultipleSelectValue();
                }
            };

            $scope.field.setValue = function(value) {
                this.widget.currentRow[this.name] = value;
                handleCheckedState();
            };

            $scope.field.value = function(value) {
                if (typeof value !== "undefined") {
                    this.widget.currentRow[this.name] = value;
                    handleCheckedState();
                    templateManager.updateTemplate();
                } else {
                    return this.widget.currentRow[this.name];
                }
            };

            $scope.toggleValue = toggleValue;

            handleCheckedState();

            if (!input.is(':disabled')) {
                label.find(".zh-box").keypress(function(e) {
                    e.preventDefault();
                    toggleValue();
                });
            }
        }
    };
};

var ZeedhiFields = angular.module("ZeedhiFields");
ZeedhiFields.directive('infSelectMultipleSelect', infSelectMultipleSelectDirective);
ZeedhiFields.directive('infSelectMultipleCheckbox', infSelectMultipleCheckboxDirective);
