window.templateUrls.push({
    "package": "inf-select-multiple",
    "baseUrl": "bower_components/inf-select-multiple/dist/templates/"
});

function InfSelectMultipleController($scope, DecodeValueFieldService) {
    var field = $scope.field;

    var defaultBuildParams = {
        fields: field.fields,
        fieldsSeparator: '-',
        valuesSeparator: '_'
    };
    var buildParams = field.concatenatedValueString ? field.concatenatedValueString : defaultBuildParams;
    buildParams.fields = Util.isArray(buildParams.fields) ? buildParams.fields : defaultBuildParams.fields;
    buildParams.fieldsSeparator = !Util.isArray(buildParams.fieldsSeparator) ? buildParams.fieldsSeparator : defaultBuildParams.fieldsSeparator;
    buildParams.valuesSeparator = !Util.isArray(buildParams.valuesSeparator) ? buildParams.valuesSeparator : defaultBuildParams.valuesSeparator;

    field.valueStrBuildParams = buildParams;

    field.isSelectReadOnly = function() {
        return field.checked || field.isReadOnly();
    };

    field.createFilter = function() {
        if (buildParams && buildParams.fields && buildParams.fields.length) {
            var valueStr = field.value();

            if (valueStr != field.valueToAll) {
                var valuesSeparator = buildParams.valuesSeparator;
                var valueArray = typeof(valueStr) == 'string' ? valueStr.split(valuesSeparator) : [];
                var filterName = field.name;

                var filter = {
                    "name": filterName,
                    "operator": 'IN',
                    "value": valueArray
                };
                return [filter];
            } else {
                return [];
            }
        } else {
            return [];
        }
    };

    var decodeInfSelectMultiple = function(field, fieldRow) {
        if (fieldRow === field.valueToAll) {
            return field.checkboxLabel;
        } else {
            let valueParams = field.concatenatedValueString;
            if (valueParams && valueParams.fields && valueParams.valuesSeparator) {
                let valueFields = valueParams.fields;
                let valuesSeparator = valueParams.valuesSeparator;
                let fieldsSeparator = valueParams.fieldsSeparator;

                let getValueFromRow = function(row) {
                    if (valueFields.length == 1) {
                        return row[valueParams.fields[0]];
                    }
                    if (valueFields.length > 1 && fieldsSeparator) {
                        return valueFields.map(function(valueField) {
                            return row[valueField];
                        }).join(fieldsSeparator);
                    }
                    return null;
                };
                let isValueSelected = function(value) {
                    let selectedValues = fieldRow.split(valuesSeparator);
                    return (selectedValues.indexOf(value) != -1);
                };
                let isRowSelected = function(row) {
                    let rowValue = getValueFromRow(row);
                    return ((rowValue !== null) && isValueSelected(rowValue));
                };
                let getRowDescription = function(row) {
                    return row[field.descriptionField];
                };

                let selectedRows = field.dataSource.data.filter(isRowSelected);
                let selectedRowsDescriptions = selectedRows.map(getRowDescription);

                return selectedRowsDescriptions.join(", ");
            }
        }
        return null;
    };
    DecodeValueFieldService.register('inf-select-multiple.html', decodeInfSelectMultiple);

}

//SELECT
function InfSelectMultipleSelectController($scope, $rootScope, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, metaDataFactory, EventChildPropagation) {

    var buildMultipleSelectValueStr = function(field, rows, buildParams) {
        var valuesArray = [];
        rows.forEach(function(item) {
            var itemArray = [];
            buildParams.fields.forEach(function(field) {
                itemArray.push(item[field]);
            });
            var itemString = itemArray.join(buildParams.fieldsSeparator);
            valuesArray.push(itemString);
        });
        field.valueAsArray = valuesArray;
        return valuesArray.length ? valuesArray.join(buildParams.valuesSeparator) : null;

    };

    function overrideSelectWidgetUnselectAllFunction(field) {
        var originalFunction = field.selectWidget.unSelectAll;
        field.selectWidget.unSelectAll = function() {
            originalFunction();
        };
    }

    var _openField = function(field) {

        var defer = ZHPromise.defer();
        var actions = [{
            "label": ScreenService.i18n("Cancel"),
            "showAsAction": "back",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "hideIcon": true,
            "events": [{
                "name": "ActionEvent",
                "code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
                "id": 9998
			}]
		}, {
            "label": ScreenService.i18n("Ok"),
            "showAsAction": "never",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "events": [{
                "name": "ActionEvent",
                "code": "args.owner.widget.setSelected(args.owner.widget.dataSource.getCheckedRows(), args.owner.widget.selectField); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
                "id": 9999
			}]
		}];

        if (field.showAddButtonAs == 'action') {
            actions.push({
                "icon": "plus",
                "showAsAction": "always|edit|checked_rows|view",
                "isVisible": true,
                "showOnForm": true,
                "showOnList": true,
                "events": [{
                    "name": "ActionEvent",
                    "code": "args.owner.widget.onAdd()",
                    "id": 9999
				}]
            });
        }

        innerSearch("", field);

        var template;
        if (!field.showSelectAs) {
            template = "widget/grid.html";
        } else if (field.showSelectAs === "tree") {
            template = "widget/tree/grid.html";
        } else {
            template = "widget/" + field.showSelectAs + ".html";
        }

        var fields;
        if (field.fields && field.fields.length) {
            fields = field.fields;
        } else {
            fields = [{
                "label": field.sourceLabel,
                "name": field.descriptionField,
                "showOnList": true
			}];
        }

        var updateCheckedRows = function() {
            if (field && field.multipleSelectCheckedRows) {
                field.dataSource.checkedRows = field.multipleSelectCheckedRows;
            }
            defer.resolve();
        };

        var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
        if (!field.selectWidget) {
            var widgetEvents = [{
                "name": "WidgetAfterMoveRow",
                "code": "args.owner.toggle(args.row, args.owner);",
                "id": 9999
			}];
            field.selectWidget = metaDataFactory.widgetFactory({
                "id": 9999,
                "name": "select_multiple_widget",
                "label": ScreenService.i18n(field.label),
                "template": template,
                "container": field.widget.container,
                "isVisible": true,
                "popupNoBlock": false,
                "showDescriptionOrder": false,
                "floatingControl": {
                    "openSearch": !Util.isMobile(),
                    "defaultSearchField": field.defaultSearchField || field.descriptionField,
                    "pdfAction": false,
                    "xlsAction": false,
                    "csvAction": false,
                    "searchAction": true,
                    "customizationAction": {
                        "filter": false,
                        "layout": false,
                        "view": false
                    }
                },

                "parentProperty": field.parentProperty,
                "primaryProperty": field.primaryProperty,
                "expandingProperty": field.expandingProperty,
                "breadcrumbProperty": field.breadcrumbProperty,
                "selectCascade": field.selectCascade,
                "childrenDependency": field.childrenDependency,

                "order": field.order,
                "reverse": field.reverse,
                "bringGroupsCollapsed": field.bringGroupsCollapsed,
                "selectField": field,
                "showCheckbox": true,
                "onAdd": field.onAdd,
                "showAddButtonAs": field.showAddButtonAs,
                "addButtonText": field.addButtonText,
                "noMatchText": field.noMatchText,
                "searchNoMatchText": field.searchNoMatchText,
                "itemsPerPage": field.itemsPerPage,
                "searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
                "hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
                "onDemand": field.onDemand,
                "searchCriteria": field.searchCriteria,

                "fields": fields,
                "dataSource": field.dataSource,
                "dataSourceFilter": field.dataSourceFilter,
                "events": EventChildPropagation.propagate(widgetEvents, field),
                "actions": actions,

                "styles": field.selectWidgetStyles
            });
            field.selectWidget.setSelected = setSelected;
            field.selectWidget.afterClosePopup = afterClosePopup;
            isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
            overrideSelectWidgetUnselectAllFunction(field);
        }

        if (isToReload) {
            field.selectWidget.searchCriteria = {};
            field.selectWidget.searchList = "";
            field.reload().then(updateCheckedRows, updateCheckedRows);
        } else {
            updateCheckedRows();
        }

        ScreenService.openPopup(field.selectWidget);
        return defer.promise;
    };

    $scope.openField = function(field) {
        var e = false;
        if (!field.isSelectReadOnly()) {
            if (field.beforeSelectOpen) {
                e = field.beforeSelectOpen();
                if (e.isDefaultPrevented()) {
                    return;
                }
            }
            // remove focus from element
            field.element.blur();
            window.getSelection().removeAllRanges(); //bug webkit

            _openField(field).then(function() {
                if (field.afterSelectOpen) {
                    field.afterSelectOpen();
                }
            });
        }
    };

    function innerSearch(selectListSearch, field) {
        if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
            var dataSource = field.dataSource;
            var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
            dataSource.filter(filter, function(data) {
                dataSource.data = data;
                templateManager.updateTemplate();
            });
        }
    }

    var setChanged = function(field) {
        field.widget.currentRow.changed = true;
    };

    function setSelected(rows, field) {
        ScreenService.closePopup();

        field.multipleSelectValueStr = buildMultipleSelectValueStr(field, rows, field.valueStrBuildParams);
        field.multipleSelectCheckedRows = rows;
        changeByValueStr(field);
        setChanged(field);

        if (field._afterSelected) {
            field._afterSelected(row);
        }

        if (field.change) field.change();
    }

    function afterClosePopup(field) {
        if (field.element) {
            field.element.focus();
        }
    }

    function changeByValueStr(field) {
        field.widget.currentRow[field.name] = field.multipleSelectValueStr;
    }

    $scope.maintainWindow = function(field) {
        if (field.maintainWindow) {
            var maintainWindowFilter = SelectService.buildMaintainFilter(field);
            ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
        }
    };

    $scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);

    function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
        if ($scope.field.selectWidget) {
            $scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
        }
    }

    this.hasSelectedRows = function hasSelectedRows(field) {
        var selectedRows = field.value();
        return selectedRows && selectedRows.length > 0;
    };
}
var infSelectMultipleSelectDirective = function($compile, $timeout, templateManager, Query, ZHPromise, ZhUtil) {
    return {
        priority: 1,
        link: function($scope, element) {
            // these options must be replaced by dataSource values
            var options = [];
            var field = $scope.field;

            var buildMultipleSelectValueStr = function(field, rows, buildParams) {
                var valuesArray = [];
                rows.forEach(function(item) {
                    var itemArray = [];
                    buildParams.fields.forEach(function(field) {
                        itemArray.push(item[field]);
                    });
                    var itemString = itemArray.join(buildParams.fieldsSeparator);
                    valuesArray.push(itemString);
                });
                return valuesArray.length ? valuesArray.join(buildParams.valuesSeparator) : null;

            };

            var getValueField = function() {
                return field.valueField ? field.valueField : field.valueFields;
            };

            var getDescriptionField = function() {
                return field.descriptionField;
            };

            element.on('keydown', function(e) {
                return e.ctrlKey || e.keyCode == 9 || (e.keyCode >= 112 && e.keyCode <= 126); // TAB key
            });

            element.on('input', function(e) {
                element.text("");
                return false;
            });

            $scope.blurInput = function() {
                element.blur();
            };

            $scope.removeOption = function(evt) {
                var elm = angular.element(evt.currentTarget);
                if (!field.isReadOnly() && !field.isSelectReadOnly()) {
                    var row = options.splice(elm.index(), 1);
                    field.dataSource.removeCheckedRows(row);
                    var checkedRows = field.dataSource.getCheckedRows();
                    field.multipleSelectValueStr = buildMultipleSelectValueStr(field, checkedRows, field.valueStrBuildParams);
                    field.multipleSelectCheckedRows = checkedRows;
                    $scope.fieldRow = field.multipleSelectValueStr;
                    elm.animate({
                        opacity: 0
                    }, 300, function() {
                        elm.animate({
                            width: 0
                        }, 300, buildOptions);
                    });
                    field.widget.currentRow.changed = true;
                    if (field.change) {
                        field.change();
                    }
                }
                evt.stopPropagation();
            };

            function buildOptions() {
                var container = element.find('.zh-options-container');
                var elementWidth = element.width();
                var optionsWidth = 0;
                var dField = getDescriptionField();

                // clear options added to element
                container.html('');

                // add options to element until options width be greater than element width

                options.forEach(function(item, index) {
                    var option;
                    if (field.isReadOnly()) {
                        option = $compile('<span class="zh-select-item" contenteditable="false"><span>&times;</span>' + item[dField] + '</span>')($scope);
                    } else {
                        option = $compile('<span class="zh-select-item" contenteditable="false" zh-touchstart="removeOption(evt)"><span>&times;</span>' + item[dField] + '</span>')(
                            $scope);
                    }
                    container.append(option);
                    optionsWidth += option[0].clientWidth + 5;
                    if (optionsWidth > elementWidth) {
                        return false;
                    }
                });

                // adjust the options and add the "more" element
                if (optionsWidth > elementWidth) {
                    var more = $compile('<span class="zh-select-item more-items">' + (options.length - container.children().length) + ' mais...</span>')($scope);
                    container.append(more);
                    optionsWidth += more[0].clientWidth + 5;
                    while (optionsWidth > elementWidth && container.children().length > 1) {
                        var lastOption = container.children().eq(-2);
                        optionsWidth -= lastOption[0].clientWidth + 5;
                        lastOption.remove();
                        more.html((options.length - container.children().length + 1) + ' mais...');
                    }
                }
            }

            // bind the window.resize event to rebuild the options if element width changes
            $(window).on("resize.select-multiple", buildOptions);

            function buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows) {
                var vField = getValueField();

                var idsByKey = {};
                if (Util.isArray(vField)) {
                    vField.forEach(function(key) {
                        idsByKey[key] = [];
                    });
                    nonMatchedRows.forEach(function(row) {
                        vField.forEach(function(key) {
                            idsByKey[key].push(row[key]);
                        });
                    });
                } else {
                    idsByKey[vField] = nonMatchedRows;
                }

                var filters = ZhUtil.buildFiltersInField(field);
                var query = Query.build();

                Object.keys(idsByKey).forEach(function(key) {
                    query.where(key).in(idsByKey[key]);
                });

                return filters.concat(query.where());
            }

            function alreadyHasTheRowInCache(row) {
                return field.dataSource.getCheckedRows().some(function(cachedRow) {
                    return Util.parseToArray(getValueField()).every(function(key) {
                        return cachedRow[key] == row[key];
                    });
                });
            }

            function alreadyHasTheRowInTheOptions(row) {
                return options.some(function(option) {
                    return Util.parseToArray(getValueField()).every(function(key) {
                        return option[key] == row[key];
                    });
                });
            }

            function addOption(row) {
                if (!alreadyHasTheRowInTheOptions(row)) {
                    options.push(row);
                    field.dataSource.addCheckedRows(row);
                }
            }

            function loadUnmatchedRowsInDatasource(nonMatchedRows) {

                var filters = buildLoadUnmatchedRowsInDatasourceFilter(nonMatchedRows);

                return field.dataSource.filter(filters).then(function(data) {
                    if (Array.isArray(data) && data.length) {
                        data.forEach(addOption);
                    }
                });
            }

            // built options property checking the values (selectedRows) against the datasource
            function buildOptionsFromDataSource(selectedRows) {
                var defer = ZHPromise.defer();

                var vField = getValueField();
                var previousCheckedRows = field.dataSource.getCheckedRows();
                field.dataSource.clearCheckedRows();
                options = [];

                if (field.dataSource.data && field.dataSource.data.length && field.multipleSelectValueStr) {
                    var buildParams = field.valueStrBuildParams;
                    var valueFields = buildParams.fields;
                    var selectDataSource = field.selectWidget.dataSource.data;

                    var valuesArray = field.multipleSelectValueStr.split(buildParams.valuesSeparator);
                    var selectedRowsArr = [];

                    valuesArray.forEach(function(valueString, valueIndex) {
                        selectedRowsArr.push({});
                        var valueArray = valueString.split(buildParams.fieldsSeparator);
                        valueFields.forEach(function(valueField, fieldIndex) {
                            selectedRowsArr[valueIndex][valueField] = valueArray[fieldIndex];
                        });
                    });

                    var filterDataSource = function(dataSourceRow) {
                        for (var i in selectedRowsArr) {
                            var selectedRow = selectedRowsArr[i];
                            var isSelected = true;
                            for (let j = 0; j < valueFields.length; j++) {
                                let dataSourceRowValue = String(dataSourceRow[valueFields[j]]);
                                let selectedRowValue = String(selectedRow[valueFields[j]]);
                                if (dataSourceRowValue !== selectedRowValue) {
                                    isSelected = false;
                                    break;
                                }
                            }
                            if (isSelected) {
                                return true;
                            }
                        }
                        return false;
                    };

                    options = selectDataSource.filter(filterDataSource);
                    field.dataSource.checkedRows = options;
                    defer.resolve();
                } else {
                    options = previousCheckedRows;
                    defer.resolve();
                }
                return defer.promise;
            }

            function findRowFromDataSourceOrCheckedRows(dataSourceData, checkedRows, valueField, valueItem) {
                return findInItems(valueField, valueItem, dataSourceData) || findInItems(valueField, valueItem, checkedRows);
            }

            function findInItems(valueField, valueItem, items) {
                return items.filter(function(item) {
                    if (Util.isArray(valueField)) {
                        return valueField.every(function(key) {
                            return item[key] == valueItem[key];
                        });
                    }
                    return item[valueField] == valueItem;
                }).shift();
            }

            var dataSourceWatch = $scope.$watch(function() {
                return field.dataSource.data;
            }, function() {
                if (field.dataSource.data && field.dataSource.data.length) {
                    dataSourceWatch();
                    buildOptionsFromDataSource((field.value() || [])).then(function() {
                        buildOptions();
                    });
                }
            });

            $scope.$watch(function() {
                return field.value();
            }, function(value) {
                buildOptionsFromDataSource(value).then(function() {
                    buildOptions();
                });
            });
        }
    };
};

//CHECKBOX
var infSelectMultipleCheckboxDirective = function(templateManager) {
    return {
        priority: 1,
        link: function($scope, $element) {
            var field = $scope.field;
            var input = $element.find("input");
            var label = $element.find("label");

            field.valueToAll = Util.isUndefined(field.valueToAll) ? "T" : field.valueToAll;

            var getMultipleSelectValue = function() {
                return field.multipleSelectValueStr ? field.multipleSelectValueStr : null;
            };
            var clearMultipleSelectValue = function() {
                field.multipleSelectValueStr = null;
                field.multipleSelectCheckedRows = [];
                if (field.dataSource && field.dataSource.clearCheckedRows) {
                    field.dataSource.clearCheckedRows();
                }
            };

            function toggleValue() {
                if (!input.is(':disabled')) {
                    field.checked = !field.checked;

                    if (!field.readOnly) {
                        $scope.widget.currentRow[field.name] = field.checked ? field.valueToAll : getMultipleSelectValue();
                        $scope.widget.currentRow.changed = true;

                        if (field.change) {
                            field.change();
                        }

                        templateManager.onUpdate();
                    }

                }

                handleCheckedState();
            }

            var handleCheckedState = function() {
                var currentValue = $scope.widget.currentRow[field.name];
                if (currentValue === field.valueToAll) {
                    field.checked = true;
                    clearMultipleSelectValue();
                } else if (field.checked) {
                    currentValue = field.valueToAll;
                    clearMultipleSelectValue();
                } else if (currentValue === getMultipleSelectValue()) {
                    field.checked = false;
                } else {
                    field.checked = false;
                    currentValue = getMultipleSelectValue();
                }
            };

            $scope.field.setValue = function(value) {
                this.widget.currentRow[this.name] = value;
                handleCheckedState();
            };

            $scope.field.value = function(value) {
                if (typeof value !== "undefined") {
                    this.widget.currentRow[this.name] = value;
                    handleCheckedState();
                    templateManager.updateTemplate();
                } else {
                    return this.widget.currentRow[this.name];
                }
            };

            $scope.toggleValue = toggleValue;

            handleCheckedState();

            if (!input.is(':disabled')) {
                label.find(".zh-box").keypress(function(e) {
                    e.preventDefault();
                    toggleValue();
                });
            }
        }
    };
};

var ZeedhiFields = angular.module("ZeedhiFields");
ZeedhiFields.directive('infSelectMultipleSelect', infSelectMultipleSelectDirective);
ZeedhiFields.directive('infSelectMultipleCheckbox', infSelectMultipleCheckboxDirective);
